<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogCategoryLocalization
{
    public function __construct(
        public string $name,
        public ?string $slug,
    ) {}
}
