<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogProductSource
{
    public function __construct(
        public string $instance_id,
        public string $product_id,
        public string $product_updated_at,
        public string $snapshot_hash,
    ) {}
}
