<?php

namespace CpCreation\VitiCore\Behavior;

interface HasGeocoder
{
    public function getAddress(): ?string;

    public function setAddress(?string $address): void;

    public function getPostal(): ?string;

    public function setPostal(?string $postal): void;

    public function getCity(): ?string;

    public function setCity(?string $city): void;

    public function getCountry(): ?string;

    public function setCountry(?string $country): void;

    public function getLat(): ?string;

    public function setLat(?string $lat): void;

    public function getLng(): ?string;

    public function setLng(?string $lng): void;
}
