{% extends 'base.html.twig' %}

{% block title %}{{ 'news.title'|trans({}, 'meta') }}{% endblock %}

{% block meta %}
    {{ include('partials/seo.html.twig', {
        title_key: 'news.title',
        description_key: 'news.description',
        robots: 'index,follow'
    }) }}
{% endblock meta %}

{% block main %}
    <section class="bg-page px-4 py-16 sm:px-8 sm:py-20">
        <div class="mx-auto max-w-large">
            <twig:news:Detail :slug="slug|default(app.request.attributes.get('slug'))" />
        </div>
    </section>
{% endblock %}
