<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Tracking;

/**
 * @param array<string, mixed> $payload
 * @param array<string, mixed> $context
 */
final readonly class ServerTrackingMessage
{
    public const string PROVIDER_MATOMO = 'matomo';
    public const string PROVIDER_META = 'meta';

    public function __construct(
        public string $provider,
        public array $payload,
        public array $context = [],
    ) {
    }
}
