<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Commerce\Catalog;

final class CatalogContract
{
    public const string SCHEMA_VERSION = '1.0';

    public const string USER_AGENT = 'Serenity-Commerce/' . self::SCHEMA_VERSION;

    private function __construct() {}
}
