<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogCommerceConstraints
{
    public function __construct(
        public string $currency,
        public CatalogCountryRules $country_rules,
    ) {}
}
