<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Product;

use CpCreation\VitiCore\Product\DTO\DegressiveContext;
use CpCreation\VitiCore\Product\DTO\DegressiveResult;

interface DegressiveCalculatorInterface
{
    public function calculate(DegressiveContext $context): ?DegressiveResult;
}
