<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Commerce\Catalog\Api\State;

use ApiPlatform\Metadata\Operation;
use ApiPlatform\State\ProviderInterface;
use CpCreation\VitiCore\Commerce\Catalog\Api\Resource\CatalogInstanceResource;
use CpCreation\VitiCore\Commerce\Catalog\CatalogInstanceProjector;

/**
 * @implements ProviderInterface<CatalogInstanceResource>
 */
final readonly class CatalogInstanceProvider implements ProviderInterface
{
    public function __construct(private CatalogInstanceProjector $projector) {}

    public function provide(Operation $operation, array $uriVariables = [], array $context = []): CatalogInstanceResource
    {
        return $this->projector->project();
    }
}
