{% extends 'base.html.twig' %}

{% block title %}{{ 'cgv.title'|trans({}, 'meta') }}{% endblock %}

{% block meta %}
    {{ include('partials/seo.html.twig', {
        title_key: 'cgv.title',
        description_key: 'cgv.description',
        robots: 'noindex,nofollow'
    }) }}
{% endblock meta %}

{% block main %}
    <twig:legal:Page type="cgv" />
{% endblock %}
