<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogCountryRules
{
    /**
     * @param list<string>|null $allowed
     * @param list<string>      $denied
     */
    public function __construct(
        public ?array $allowed,
        public array $denied,
    ) {}
}
