<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\SocialPublishing;

final readonly class Publication
{
    /** @param list<PublicationTarget> $targets */
    public function __construct(
        public string $id,
        public string $message,
        public ?string $mediaUrl,
        public \DateTimeImmutable $scheduledAt,
        public string $status,
        public ?string $error,
        public ?\DateTimeImmutable $publishedAt,
        public array $targets,
    ) {}
}
