<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogAvailability
{
    /**
     * @param list<string> $reasons
     */
    public function __construct(
        public string $status,
        public bool $orderable,
        public bool $tracked,
        public ?int $quantity,
        public string $as_of,
        public array $reasons,
    ) {}
}
