<section {{ attributes.defaults({ class: 'mx-auto mt-12 mb-12 w-full md:mt-16 md:mb-16' }) }}>
    <div class="border border-foreground/10 bg-surface px-4 py-4 shadow-sm md:px-10">
        <div class="flex flex-col gap-5 sm:flex-row sm:items-center">
            <div class="flex shrink-0 justify-center sm:justify-start">
                <img
                    src="{{ asset('bundles/cpcreationviticore/images/prevention-banner.jpg') }}"
                    alt="{{ 'shop.legal_notice.image_alt'|trans }}"
                    class="h-24 w-24 object-contain md:h-28 md:w-28">
            </div>
            <div class="space-y-3 text-center sm:text-left">
                <p class="text-xs font-medium text-foreground md:text-sm">
                    {{ 'shop.legal_notice.text'|trans }}
                </p>
                <p class="text-xs font-semibold text-foreground/80 md:text-sm">
                    {{ 'shop.legal_notice.code'|trans }}
                </p>
                <p class="text-center text-xs text-foreground/75 md:text-sm">
                    {{ 'shop.legal_notice.moderation'|trans }}
                </p>
            </div>
        </div>
    </div>
</section>
