<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Marketing;

use CpCreation\VitiCore\Marketing\Model\AbandonedCartReminder;

final readonly class IssuedAbandonedCartReminder
{
    public function __construct(
        public AbandonedCartReminder $reminder,
        public string $token,
    ) {}
}
