{% if template %}
    {% include template with { vacation: this } only %}
{% elseif this.active and config.vacation|default('')|striptags|trim is not empty %}
    <section {{ attributes.defaults({ class: 'mx-auto mt-10 w-full max-w-7xl px-4 md:px-10 lg:mt-20' }) }} role="status">
        <div class="border border-foreground/15 bg-surface px-5 py-6 text-center text-sm leading-7 text-foreground shadow-sm md:px-10 md:text-base">
            {{ config.vacation|raw }}
        </div>
    </section>
{% endif %}
