<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Mailer;

use Symfony\Component\Messenger\Stamp\StampInterface;

final readonly class EmailLogStamp implements StampInterface
{
    public function __construct(
        public string $emailLogId,
    ) {
    }
}
