<?php

namespace CpCreation\VitiCore\Behavior;

use CpCreation\VitiCore\Cart\Model\PaymentMethod;

interface HasPayment
{
    public function getPaymentMethod(): ?PaymentMethod;

    public function getTotal(): ?int;

    public function getToken(): ?string;

    public function getEmail(): ?string;

    public function getFirstname(): ?string;

    public function getLastname(): ?string;

    public function getPhone(): ?string;

    public function getCompany(): ?string;

    public function getStreet(): ?string;

    public function getPostalCode(): ?string;

    public function getCity(): ?string;

    public function getCountry(): ?string;

    public function getQuantity(): int;
}
