{{ 'promo.mail.intro'|trans({'%company%': company }, 'messages', locale|default('fr')) }}

{{ 'promo.mail.code_label'|trans({}, 'messages', locale|default('fr')) }}: {{ promo.title }}
{{ 'promo.mail.benefit_label'|trans({}, 'messages', locale|default('fr')) }}: {{ benefit_label }}
{{ 'promo.mail.validity_label'|trans({}, 'messages', locale|default('fr')) }}: {{ validity_label }}
{% if condition_lines is not empty %}

{{ 'promo.mail.conditions_label'|trans({}, 'messages', locale|default('fr')) }}:
{% for condition in condition_lines %}
- {{ condition }}
{% endfor %}
{% endif %}

{% if promo.assignedUser and promo.auto %}
{{ 'promo.mail.auto_notice'|trans({}, 'messages', locale|default('fr')) }}
{% else %}
{{ 'promo.mail.manual_notice'|trans({}, 'messages', locale|default('fr')) }}
{% endif %}

{{ 'promo.mail.footer'|trans({'%company%': company }, 'messages', locale|default('fr')) }}
