<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Marketing;

use CpCreation\VitiCore\Product\Model\Product;

interface PublishedProductReviewRatingProvider
{
    public function getPublishedRatingSummary(
        Product $product,
        ?\DateTimeImmutable $now = null,
    ): ?ProductReviewRatingSummary;
}
