<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\SocialPublishing;

final readonly class PublicationTarget
{
    public function __construct(
        public Channel $channel,
        public string $status,
        public ?string $externalPublicationId,
        public ?string $error,
    ) {}
}
