<?php

declare(strict_types=1);

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Product;

use CpCreation\VitiCore\Cart\Model\Cart;
use CpCreation\VitiCore\Product\DTO\DegressiveResult;

final class DegressiveManager
{
    public function __construct(
        private DegressiveContextFactory $degressiveContextFactory,
        private DegressiveCalculatorInterface $degressiveCalculator,
    ) {
    }

    /**
     * @throws \Doctrine\ORM\NonUniqueResultException
     * @throws \Exception
     */
    public function calculateForCart(Cart $cart): ?DegressiveResult
    {
        $context = $this->degressiveContextFactory->createFromCart($cart);

        if (null === $context) {
            return null;
        }

        return $this->degressiveCalculator->calculate($context);
    }
}
