<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogSellerAddress
{
    public function __construct(
        public ?string $street,
        public ?string $postal_code,
        public ?string $city,
        public string $country,
    ) {}
}
