{% extends 'base.html.twig' %}

{% block title %}{{ 'news.title'|trans({}, 'meta') }}{% endblock %}

{% block meta %}
    {{ include('partials/seo.html.twig', {
        title_key: 'news.title',
        description_key: 'news.description',
        robots: 'index,follow'
    }) }}
{% endblock meta %}

{% block main %}
    <section class="bg-page px-4 py-16 sm:px-8 sm:py-20">
        <div class="mx-auto max-w-large">
            <div class="mx-auto mb-10 max-w-4xl px-6 text-center sm:px-8">
                <h1 class="flex flex-col items-center font-serif text-3xl uppercase tracking-label text-foreground sm:text-4xl lg:text-5xl after:mt-5 after:h-0.5 after:w-14 after:bg-accent">
                    {{ 'news.title'|trans({}, 'meta') }}
                </h1>
                <p class="mt-5 text-sm leading-7 text-foreground/70 sm:text-base">{{ 'news.intro'|trans }}</p>
            </div>

            <twig:news:SummaryList />
        </div>
    </section>
{% endblock %}
