<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogProductReview
{
    public function __construct(
        public string $id,
        public int $rating,
        public string $author_display_name,
        public ?string $title,
        public string $comment,
        public bool $verified_purchase,
        public string $purchased_at,
        public string $published_at,
        public ?string $merchant_reply,
    ) {}
}
