<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Marketing;

final readonly class AbandonedCartDispatchResult
{
    public function __construct(
        public int $eligibleCount,
        public int $createdCount,
        public int $skippedCount,
    ) {}
}
