<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Cart;

use CpCreation\VitiCore\Cart\Model\Cart;

/**
 * @param list<Cart> $eligibleCarts
 * @param list<Cart> $markedCarts
 */
final readonly class CartAbandonmentResult
{
    public function __construct(
        public \DateTimeImmutable $markedAt,
        public \DateTimeImmutable $pendingCutoff,
        public \DateTimeImmutable $checkoutCutoff,
        public array $eligibleCarts,
        public array $markedCarts,
        public int $markedCount,
        public bool $dryRun,
    ) {
    }
}
