<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Enrollment\Message;

final readonly class DispatchCatalogEnrollmentsMessage
{
    public function __construct(public int $limit = 25) {}
}
