<?php

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Altcha;

use CpCreation\AltchaBundle\Contract\ScopeProviderInterface;
use CpCreation\AltchaBundle\Model\ScopeDefinition;

final class VitiCoreAltchaScopeProvider implements ScopeProviderInterface
{
    public function scopes(): iterable
    {
        yield new ScopeDefinition(VitiCoreAltchaProtection::CONTACT, 'Contact');
        yield new ScopeDefinition(VitiCoreAltchaProtection::REGISTRATION_CONFIRM, 'Confirmation d’inscription');
        yield new ScopeDefinition(VitiCoreAltchaProtection::PASSWORD_RESET, 'Réinitialisation du mot de passe');
        yield new ScopeDefinition(VitiCoreAltchaProtection::PAYMENT, 'Paiement en ligne');
        yield new ScopeDefinition(VitiCoreAltchaProtection::PRODUCT_REVIEW_REPORT, 'Signalement d’un avis');
        yield new ScopeDefinition(VitiCoreAltchaProtection::WITHDRAWAL, 'Rétractation');
    }
}
