<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Commerce\Catalog\Configuration;

final readonly class CatalogConfigurationSnapshot
{
    public function __construct(
        public bool $enabled,
        public ?string $hubUrl,
        public string $instanceId,
        public string $currency,
        public string $authAudience,
        #[\SensitiveParameter]
        public ?string $enrollmentToken,
        #[\SensitiveParameter]
        public ?string $webhookSecret,
        public ?string $webhookUrl,
        public ?string $jwtIssuer,
        public ?string $jwtKeyId,
        public ?string $jwtPublicKeyPem,
        public bool $monitoringEnabled = false,
    ) {}
}
