<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\User;

final readonly class ExpiredRegistrationPurgeResult
{
    public function __construct(
        public int $eligibleCount,
        public int $deletedCount,
        public int $skippedCount,
        public int $emailLogsDeleted,
        public int $auditRowsDeleted,
        public int $batchCount,
        public bool $hasMore,
    ) {
    }
}
