<!DOCTYPE html>
<html lang="fr">
<head>
    <meta charset="UTF-8"/>
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1">
    <title>{% block title %}{{ 'admin.brand'|trans }}{% endblock %}</title>
    <link rel="stylesheet" href="{{ asset('bundles/cpcreationviticore/css/admin/lib/bootstrap.min.css') }}">
    <link rel="stylesheet"
          href="{{ asset('bundles/cpcreationviticore/css/admin/lib/bootstrap-datetimepicker.min.css') }}">
    <link rel="stylesheet" href="{{ asset('bundles/cpcreationviticore/css/admin/lib/metisMenu.min.css') }}">
    <link rel="stylesheet" href="{{ asset('bundles/cpcreationviticore/css/admin/lib/sb-admin-2.min.css') }}">
    <link rel="stylesheet" href="{{ asset('bundles/cpcreationviticore/css/admin/lib/trumbowyg.min.css') }}">
    <link rel="stylesheet" href="{{ asset('bundles/cpcreationviticore/css/admin/lib/select2.min.css') }}">
    <link rel="stylesheet" href="{{ asset('bundles/cpcreationviticore/css/admin/lib/wizardflow.css') }}">
    <link rel="stylesheet" href="{{ asset('bundles/cpcreationviticore/css/admin/lib/datatables.min.css') }}">
    <link rel="stylesheet" href="{{ asset('bundles/cpcreationviticore/css/admin/main.css?v20260507') }}">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.11.2/css/all.css"
          crossorigin="anonymous">
    {% block stylesheets %}{% endblock %}
</head>
<body>

<!-- /#wrapper -->
<div id="wrapper">

    <!-- Navigation -->
    <nav class="navbar navbar-default navbar-static-top" role="navigation" style="margin-bottom: 0">
        <div class="navbar-header">
            <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".navbar-collapse">
                <span class="sr-only">Toggle navigation</span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            <a class="navbar-brand" href="{{ path('admin_homepage') }}">{{ 'admin.brand'|trans }}</a>
        </div>
        <!-- /.navbar-header -->

        <ul class="nav navbar-top-links navbar-right">
            <!-- /.dropdown -->
            {% if app.user %}
                <li class="dropdown">
                    <a class="dropdown-toggle" data-toggle="dropdown" href="#">
                        <i class="fa fa-user fa-fw"></i> <i class="fa fa-caret-down"></i>
                    </a>

                    <ul class="dropdown-menu dropdown-user">
                        <li><a href="{{ path('admin_logout') }}"><i class="fa fa-user fa-fw"></i> Deconnexion</a>
                        </li>
                    </ul>

                    <!-- /.dropdown-user -->
                </li>
            {% endif %}
            <!-- /.dropdown -->
        </ul>
        <!-- /.navbar-top-links -->

        <div class="navbar-default sidebar" role="navigation">
            <div class="sidebar-nav navbar-collapse">
                <ul class="nav" id="side-menu">
                    {% if app.user %}
                        <li>
                            <a href="{{ path('admin_homepage') }}"><i class="fas fa-home"></i> Accueil</a>
                        </li>
                        {% if is_granted('ROLE_ORDER') or is_granted('ROLE_ADMIN') %}
                            <li>
                                <a href="#"><i class="fas fa-list-ul"></i> Commandes<span class="fa arrow"></span></a>
                                <ul class="nav nav-second-level collapse">
                                    <li>
                                        <a href="{{ path('admin_order') }}"><i class="fas fa-history"></i> Historique</a>
                                    </li>
                                    {% if invoiceFeatureEnabled() %}
                                        <li>
                                            <a href="{{ path('admin_order_invoices') }}"><i class="fas fa-file-invoice"></i> Factures</a>
                                        </li>
                                    {% endif %}
                                    <li>
                                        <a href="{{ path('admin_order_stats') }}"><i class="fas fa-chart-line"></i> Statistiques</a>
                                    </li>
                                </ul>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_ADMIN') %}
                            <li>
                                <a href="#"><i class="fas fa-users"></i> Clients<span class="fa arrow"></span></a>
                                <ul class="nav nav-second-level collapse">
                                    <li>
                                        <a href="{{ path('admin_client') }}"><i class="fas fa-user-check"></i> Clients enregistrés</a>
                                    </li>
                                    <li>
                                        <a href="{{ path('admin_guest_client') }}"><i class="fas fa-user-tag"></i> {{ 'admin.client.guests.menu'|trans }}</a>
                                    </li>
                                    <li>
                                        <a href="{{ path('admin_imported_client') }}"><i class="fas fa-address-book"></i> Base clients importée</a>
                                    </li>
                                </ul>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_PAYMENT_PAYMENT') or is_granted('ROLE_ADMIN') or is_granted('ROLE_ORDER') or is_granted('ROLE_PRICE_CODE') or is_granted('ROLE_DEGRESSIVE') or is_granted('ROLE_PRODUCT_PRICE') or is_granted('ROLE_SHIPPING') or is_granted('ROLE_SHIPPING_METHOD') or is_granted('ROLE_PROMO') %}
                            <li>
                                <a href="#"><i class="fas fa-store"></i> Boutique<span class="fa arrow"></span></a>
                                <ul class="nav nav-second-level collapse">
                                    {% if is_granted('ROLE_SHIPPING') or is_granted('ROLE_ADMIN') or is_granted('ROLE_SHIPPING_METHOD') %}
                                        <li>
                                            <a href="#"><i class="fas fa-truck"></i> Livraisons<span class="fa arrow"></span></a>
                                            <ul class="nav nav-third-level collapse">
                                                {% if is_granted('ROLE_SHIPPING') or is_granted('ROLE_ADMIN') %}
                                                    <li>
                                                        <a href="#"><i class="fas fa-box"></i> Frais de port<span class="fa arrow"></span></a>
                                                        <ul class="nav nav-fourth-level collapse">
                                                            <li>
                                                                <a href="{{ path('admin_shipping_config') }}">Configuration</a>
                                                            </li>
                                                            <li>
                                                                <a href="#">Manuel <span class="fa arrow"></span></a>
                                                                <ul class="nav nav-fifth-level collapse">
                                                                    {% if is_granted('ROLE_ADMIN') %}
                                                                        <li>
                                                                            <a href="{{ path('admin_shipping_country') }}"><i class="fas fa-flag"></i> Pays frais de port</a>
                                                                        </li>
                                                                        <li>
                                                                            <a href="{{ path('admin_shipping_zone') }}"><i class="fas fa-globe"></i> Zones frais de port</a>
                                                                        </li>
                                                                    {% endif %}
                                                                    <li>
                                                                        <a href="{{ path('admin_shipping_price') }}"><i class="fas fa-box"></i> Tarifs frais de port</a>
                                                                    </li>
                                                                </ul>
                                                            </li>
                                                            {% if is_granted('ROLE_ADMIN') %}
                                                                <li>
                                                                    <a href="#">Auto <span class="fa arrow"></span></a>
                                                                    <ul class="nav nav-fifth-level collapse">
                                                                        <li>
                                                                            <a href="{{ path('admin_shipping_auto') }}"><i class="fas fa-globe"></i> Tarifs frais de port</a>
                                                                        </li>
                                                                        <li>
                                                                            <a href="{{ path('admin_shipping_auto_sup') }}"><i class="fas fa-box"></i> Supplément de transport</a>
                                                                        </li>
                                                                    </ul>
                                                                </li>
                                                            {% endif %}
                                                            <li>
                                                                <a href="{{ path('admin_shipping_additional') }}">Suppléments pour frais Auto</a>
                                                            </li>
                                                            <li>
                                                                <a href="{{ path('admin_shipping_additional_manual') }}">Suppléments pour frais Manuels</a>
                                                            </li>
                                                        </ul>
                                                    </li>
                                                {% endif %}
                                                {% if is_granted('ROLE_SHIPPING_METHOD') or is_granted('ROLE_ADMIN') %}
                                                    <li>
                                                        <a href="#"><i class="fas fa-truck-loading"></i> Méthode de livraison<span class="fa arrow"></span></a>
                                                        <ul class="nav nav-fourth-level collapse">
                                                            {% if is_granted('ROLE_ADMIN') %}
                                                                <li>
                                                                    <a href="{{ path('admin_delivery_method_category') }}">Catégorie</a>
                                                                </li>
                                                            {% endif %}
                                                            <li>
                                                                <a href="{{ path('admin_delivery_method') }}">Liste</a>
                                                            </li>
                                                        </ul>
                                                    </li>
                                                {% endif %}
                                            </ul>
                                        </li>
                                    {% endif %}
                                    {% if is_granted('ROLE_ADMIN') or is_granted('ROLE_PRICE_CODE') or is_granted('ROLE_PRODUCT_PRICE') %}
                                        <li>
                                            <a href="#"><i class="fas fa-euro-sign"></i> Grille tarifaire<span class="fa arrow"></span></a>
                                            <ul class="nav nav-third-level collapse">
                                                {% if is_granted('ROLE_ADMIN') %}
                                                    <li>
                                                        <a href="{{ path('admin_product_price_group') }}"><i class="fas fa-tags"></i> Groupe de prix</a>
                                                    </li>
                                                {% endif %}
                                                {% if is_granted('ROLE_PRICE_CODE') or is_granted('ROLE_ADMIN') %}
                                                    <li>
                                                        <a href="{{ path('admin_product_price_code') }}"><i class="fas fa-code-branch"></i> Code partenaire</a>
                                                    </li>
                                                {% endif %}
                                                {% if is_granted('ROLE_PRODUCT_PRICE') and is_granted('ROLE_ADMIN') == false %}
                                                    <li>
                                                        <a href="{{ path('admin_product') }}"><i class="fab fa-product-hunt"></i> Produits</a>
                                                    </li>
                                                {% endif %}
                                            </ul>
                                        </li>
                                    {% endif %}
                                    {% if is_granted('ROLE_PROMO') or is_granted('ROLE_ADMIN') or is_granted('ROLE_DEGRESSIVE') %}
                                        <li>
                                            <a href="#"><i class="fas fa-percent"></i> Remise<span class="fa arrow"></span></a>
                                            <ul class="nav nav-third-level collapse">
                                                {% if is_granted('ROLE_PROMO') or is_granted('ROLE_ADMIN') %}
                                                    <li>
                                                        <a href="{{ path('admin_promo') }}"><i class="fas fa-barcode"></i> {{ 'admin.coupon'|trans({}, 'messages', 'fr') }}</a>
                                                    </li>
                                                {% endif %}
                                                {% if is_granted('ROLE_DEGRESSIVE') or is_granted('ROLE_ADMIN') %}
                                                    <li>
                                                        <a href="{{ path('admin_product_degressive') }}"><i class="fas fa-arrow-down"></i> Dégressivité</a>
                                                    </li>
                                                {% endif %}
                                            </ul>
                                        </li>
                                    {% endif %}
                                    {% if is_granted('ROLE_ADMIN') %}
                                        <li>
                                            <a href="#"><i class="fas fa-credit-card"></i> Paiement<span class="fa arrow"></span></a>
                                            <ul class="nav nav-third-level collapse">
                                                <li>
                                                    <a href="{{ path('admin_product_tax') }}"><i class="fas fa-percent"></i> Taxes</a>
                                                </li>
                                                <li>
                                                    <a href="{{ path('admin_payment') }}"><i class="fas fa-credit-card"></i> Méthode de paiement</a>
                                                </li>
                                            </ul>
                                        </li>
                                    {% endif %}
                                    {% if is_granted('ROLE_ADMIN') %}
                                        <li>
                                            <a href="{{ path('admin_shop_config') }}"><i class="fas fa-cog"></i> Configuration</a>
                                        </li>
                                        <li>
                                            <a href="{{ path('admin_shop_information_block') }}"><i class="fas fa-info-circle"></i> Blocs d'information</a>
                                        </li>
                                    {% endif %}
                                    {% if is_granted('ROLE_ORDER') or is_granted('ROLE_ADMIN') %}
                                        <li>
                                            <a href="{{ path('admin_withdrawal') }}"><i class="fas fa-undo"></i> {{ 'withdrawal.admin.menu'|trans({}, 'messages', 'fr') }}</a>
                                        </li>
                                    {% endif %}
                                </ul>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_ADMIN') or is_granted('ROLE_PRODUCT') %}
                            <li>
                                <a href="#"><i class="fab fa-product-hunt"></i> Produits<span class="fa arrow"></span></a>
                                <ul class="nav nav-second-level collapse">
                                    <li>
                                        <a href="{{ path('admin_product') }}"><i class="fab fa-product-hunt"></i> Produits</a>
                                    </li>
                                    {% if is_granted('ROLE_ADMIN') %}
                                        <li>
                                            <a href="{{ path('admin_product_type') }}"><i class="fab fa-product-hunt"></i> Types de produit</a>
                                        </li>
                                        <li>
                                            <a href="{{ path('admin_product_collection') }}"><i class="fas fa-layer-group"></i> Collections</a>
                                        </li>
                                        <li>
                                            <a href="{{ path('admin_product_category') }}"><i class="fab fa-product-hunt"></i> Catégories de produits</a>
                                        </li>
                                    {% endif %}
                                    {% if is_granted('ROLE_AWARD') or is_granted('ROLE_ADMIN') %}
                                        <li>
                                            <a href="{{ path('admin_award') }}"><i class="fas fa-trophy"></i> Récompenses</a>
                                        </li>
                                    {% endif %}
                                    {% if is_granted('ROLE_ADMIN') %}
                                        <li>
                                            <a href="{{ path('admin_label') }}"><i class="fas fa-tags"></i> Labels</a>
                                        </li>
                                    {% endif %}
                                </ul>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_NEWS') or is_granted('ROLE_ADMIN') or is_granted('ROLE_SOCIAL') %}
                            <li>
                                <a href="#"><i class="fas fa-file-alt"></i> Actualités et Photos<span class="fa arrow"></span></a>
                                <ul class="nav nav-second-level collapse">
                                    {% if is_granted('ROLE_ADMIN') %}
                                        <li>
                                            <a href="{{ path('admin_news_category') }}"><i class="fas fa-file-alt"></i> Catégories d'actualités</a>
                                        </li>
                                    {% endif %}
                                    <li>
                                        <a href="{{ path('admin_news') }}"><i class="fas fa-file-alt"></i> Actualités</a>
                                    </li>
                                    {% if is_granted('ROLE_ADMIN') %}
                                        <li>
                                            <a href="{{ path('admin_library') }}"><i class="fas fa-images"></i> Galerie photos</a>
                                        </li>
                                    {% endif %}
                                    {% if is_granted('ROLE_ADMIN') or is_granted('ROLE_SOCIAL') %}
                                        <li>
                                            <a href="{{ path('admin_social') }}"><i class="fas fa-bullhorn"></i> Réseaux sociaux</a>
                                        </li>
                                    {% endif %}
                                </ul>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_PAYMENT_PAYMENT') or is_granted('ROLE_ADMIN') %}
                            <li>
                                <a href="{{ path('admin_payment_payment') }}"><i class="fas fa-euro-sign"></i> Paiement en ligne</a>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_POPUP') or is_granted('ROLE_ADMIN') %}
                            <li>
                                <a href="{{ path('admin_popup') }}"><i class="far fa-window-restore"></i> Annonces</a>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_EVENT') or is_granted('ROLE_DEPOT') or is_granted('ROLE_ADMIN') %}
                            <li>
                                <a href="#"><i class="fas fa-copy"></i> Pages<span class="fa arrow"></span></a>
                                <ul class="nav nav-second-level collapse">
                                    <li>
                                        <a href="{{ path('admin_event') }}"><i class="far fa-calendar-alt"></i> Salons</a>
                                    </li>
                                    {% if is_granted('ROLE_ADMIN') or is_granted('ROLE_DEPOT') %}
                                        <li>
                                            <a href="#"><i class="fas fa-globe"></i> Dépôts<span class="fa arrow"></span></a>
                                            <ul class="nav nav-third-level collapse">
                                                <li>
                                                    <a href="{{ path('admin_depot_category') }}"><i class="fas fa-globe"></i> Catégories de dépôts</a>
                                                </li>
                                                <li>
                                                    <a href="{{ path('admin_depot') }}"><i class="fas fa-globe"></i> Dépôts</a>
                                                </li>
                                            </ul>
                                        </li>
                                    {% endif %}
                                </ul>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_ADMIN') or is_granted('ROLE_TRACKING') %}
                            <li>
                                <a href="#"><i class="fas fa-cogs"></i> Configuration<span class="fa arrow"></span></a>
                                <ul class="nav nav-second-level collapse">
                                    {% if is_granted('ROLE_ADMIN') %}
                                        <li>
                                            <a href="{{ path('admin_contact') }}"><i class="fas fa-building"></i> Base</a>
                                        </li>
                                        <li>
                                            <a href="{{ path('admin_age_verification') }}"><i class="fas fa-user-check"></i> {{ 'age_verification.admin.menu'|trans }}</a>
                                        </li>
                                        {% if is_granted('ROLE_SUPER_ADMIN') %}
                                            <li>
                                                <a href="{{ path('admin_commerce_configuration') }}"><i class="fas fa-project-diagram"></i> {{ 'commerce_catalog.admin.menu'|trans }}</a>
                                            </li>
                                        {% endif %}
                                    {% endif %}
                                    {% if is_granted('ROLE_ADMIN') or is_granted('ROLE_TRACKING') %}
                                        <li>
                                            <a href="{{ path('admin_tracking') }}"><i class="fas fa-chart-line"></i> Tracking</a>
                                        </li>
                                    {% endif %}
                                </ul>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_ADMIN') or is_granted('ROLE_MARKETING') %}
                            <li>
                                <a href="#"><i class="fas fa-bullhorn"></i> {{ 'marketing.admin.menu'|trans }}<span class="fa arrow"></span></a>
                                <ul class="nav nav-second-level collapse">
                                    <li>
                                        <a href="{{ path('admin_marketing') }}"><i class="fas fa-chart-pie"></i> {{ 'marketing.admin.overview'|trans }}</a>
                                    </li>
                                    <li>
                                        <a href="{{ path('admin_marketing_emailing') }}"><i class="fas fa-envelope"></i> {{ 'emailing.admin.menu'|trans }}</a>
                                    </li>
                                    <li>
                                        <a href="#"><i class="fas fa-cart-arrow-down"></i> {{ 'marketing.abandoned_cart.admin.group_menu'|trans }}<span class="fa arrow"></span></a>
                                        <ul class="nav nav-third-level collapse">
                                            <li>
                                                <a href="{{ path('admin_marketing_abandoned_cart') }}"><i class="fas fa-cog"></i> {{ 'marketing.common.configuration'|trans }}</a>
                                            </li>
                                            <li>
                                                <a href="{{ path('admin_marketing_abandoned_cart_reminders') }}"><i class="fas fa-list-alt"></i> {{ 'marketing.abandoned_cart.reminder.admin.menu'|trans }}</a>
                                            </li>
                                        </ul>
                                    </li>
                                    <li>
                                        <a href="#"><i class="fas fa-star"></i> {{ 'marketing.review.admin.group_menu'|trans }}<span class="fa arrow"></span></a>
                                        <ul class="nav nav-third-level collapse">
                                            <li>
                                                <a href="{{ path('admin_marketing_review_request') }}"><i class="fas fa-cog"></i> {{ 'marketing.common.configuration'|trans }}</a>
                                            </li>
                                            <li>
                                                <a href="{{ path('admin_marketing_product_reviews') }}"><i class="fas fa-star"></i> {{ 'marketing.review.admin.menu'|trans }}</a>
                                            </li>
                                            <li>
                                                <a href="{{ path('admin_marketing_product_review_reports') }}"><i class="fas fa-flag"></i> {{ 'marketing.review_report.admin.menu'|trans }}</a>
                                            </li>
                                        </ul>
                                    </li>
                                </ul>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_ADMIN') %}
                            <li>
                                <a href="{{ path('admin_locale') }}"><i class="fas fa-language"></i> Langues du site</a>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_ADMIN') %}
                            <li>
                                <a href="{{ path('admin_audit') }}"><i class="fas fa-clipboard-list"></i> Journal d'audit</a>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_ADMIN') %}
                            <li>
                                <a href="{{ path('admin_business_version') }}"><i class="fas fa-code-branch"></i> Versioning</a>
                            </li>
                        {% endif %}
                        {% if is_granted('ROLE_ADMIN') %}
                            <li>
                                <a href="{{ path('admin_user') }}"><i class="fas fa-user-shield"></i> Admins</a>
                            </li>
                        {% endif %}
                        {% block menus %} {% endblock %}
                    {% endif %}
                </ul>
            </div>
            <!-- /.sidebar-collapse -->
        </div>
        <!-- /.navbar-static-side -->
    </nav>

    <!-- Page Content -->
    <div id="page-wrapper">
        <div class="container-fluid">
            <div class="flashBag"></div>
            {% block content %}
                <!-- /.row -->
            {% endblock %}
        </div>
        <!-- /.container-fluid -->
    </div>
    <!-- /#page-wrapper -->

</div>
{{ include('@CpCreationVitiCore/admin/modal/confirm-deletable.html.twig') }}
{{ include('@CpCreationVitiCore/admin/modal/confirm-modification.html.twig') }}
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/jquery.min.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/bootstrap.min.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/moment.min.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/fr.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/datatables.min.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/bootstrap-datetimepicker.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/jquery.collection.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/metisMenu.min.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/sb-admin-2.min.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/trumbowyg.min.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/trumbowyg-fr.min.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/lib/select2.min.js') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/main.js?v20260528-datatable-date-sort') }}"></script>
<script src="{{ asset('bundles/cpcreationviticore/js/admin/collection.js') }}?v20260825-product-collection-positions"></script>
<script src="{{ asset('bundles/fosjsrouting/js/router.min.js') }}"></script>
<script src="{{ path('fos_js_routing_js', { callback: 'fos.Router.setData' }) }}"></script>
{% for type, flashMessages in app.session.flashbag.all() %}
    {% for flashMessage in flashMessages %}
        <script type="text/javascript">
            jQuery(".flashBag").append('<div class="alert {{ type }} alert-dismissible" role="alert">' +
                '<button type="button" class="close" data-dismiss="alert" aria-label="Close">' +
                '<span aria-hidden="true">&times;</span></button>' +
                "<strong>{{ flashMessage|raw }}</div>"
            );
        </script>
    {% endfor %}
{% endfor %}
{% block javascripts %}{% endblock %}
</body>
</html>
