<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Form\Admin;

use CpCreation\VitiCore\Cart\Model\PaymentMethod;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

final class PaymentMethodStep1Type extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        // $options['payment_methods'] = ['CB', 'Virement', ...] OU objets (à adapter)
        $choices = [];
        foreach ($options['payment_methods'] as $method) {
            // si c'est une string : label => value
            $choices[(string) $method] = $method;
        }

        $builder->add('method', ChoiceType::class, [
            'choices' => $choices,
            'label' => 'Choisir une méthode de paiement',
            'placeholder' => 'Aucun',
            'required' => false,
        ]);
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => PaymentMethod::class,
            'payment_methods' => [], // option custom
        ]);

        $resolver->setAllowedTypes('payment_methods', 'array');
    }

    public function getBlockPrefix(): string
    {
        return 'paymentMethodStep1';
    }
}
