<?php

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Bridge\Doctrine\Migrations;

use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;

final class Version20260720185000 extends AbstractMigration
{
    public function getDescription(): string
    {
        return 'Supprime le champ inutilise shipping_info de la configuration de livraison.';
    }

    public function up(Schema $schema): void
    {
        $this->addSql('ALTER TABLE shop_shipping_config_translation DROP COLUMN IF EXISTS shipping_info');
    }

    public function down(Schema $schema): void
    {
        $this->addSql('ALTER TABLE shop_shipping_config_translation ADD COLUMN IF NOT EXISTS shipping_info TEXT DEFAULT NULL');
    }
}
