<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\User\Import;

final readonly class ImportedClientStoredFile
{
    public function __construct(
        public string $filename,
        public int $size,
    ) {}
}
