<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Marketing;

final readonly class ProductReviewCardData
{
    /**
     * @param array<string, ProductReviewCardSummary> $summaries
     */
    public function __construct(
        public bool $enabled,
        public array $summaries,
    ) {}
}
