{% extends '@CpCreationVitiCore/admin/base.html.twig' %}

{% block content %}
    <div class="panel panel-default">
        <div class="panel-heading">
            Méthode de livraison
            <div class="pull-right">
                <div class="btn-group">
                    <a href="{{ path('admin_delivery_method_category_new') }}" type="button" class="btn btn-default btn-xs">
                        Ajouter
                    </a>
                </div>
            </div>
        </div>
        <table class="table table-striped">
            <thead>
            <tr>
                <th>#</th>
                <th>Nom</th>
                <th>Description</th>
                <th>Position</th>
            </tr>
            </thead>
            <tbody>
                {% for item in items %}
                    <tr>
                        <td>{{ loop.index }}</td>
                        <td>{{ item.name }}</td>
                        <td>{{ item.description }}</td>
                        <td>
                            {% if loop.first == false %}
                                <a href="{{ path('admin_delivery_method_category_move_up', {id : item.id}) }}"
                                   class="btn btn-default btn-xs" data-toggle="tooltip" title="Remonter la catégorie"><i
                                            class="fa fa-2x fa-arrow-up" aria-hidden="true"></i></a>
                            {% endif %}
                            {% if loop.last == false %}
                                <a href="{{ path('admin_delivery_method_category_move_down', {id : item.id}) }}"
                                   class=" btn btn-default btn-xs" data-toggle="tooltip" title="Descendre la catégorie"><i
                                            class="fa fa-2x fa-arrow-down" aria-hidden="true"></i></a>
                            {% endif %}
                        </td>
                        <td>
                            <a href="{{ path('admin_delivery_method_category_edit', { id: item.id}) }}" class="btn btn-info btn-xs"
                               data-title="Editer la catégorie"><i
                                        class="fas fa-edit fa-2x" aria-hidden="true"></i></a>
                                <a href="{{ path('admin_delivery_method_category_delete', { id: item.id }) }}"
                                   class="confirm-deletable btn btn-danger btn-xs"
                                   data-title="Supprimer la categorie {{ item.name }}"><i class="fa fa-2x fa-trash"
                                                                                        aria-hidden="true"></i></a>
                            {% if item.disabled == 0 %}
                                <a href="{{ path('admin_delivery_method_category_disable', { id: item.id }) }}"
                                   class=" btn btn-default btn-xs" data-toggle="tooltip"
                                   title="Dépublier la catégorie {{ item.name }}"><i
                                            class="fas fa-circle fa-2x color_green"></i></a>
                            {% else %}
                                <a href="{{ path('admin_delivery_method_category_enable', { id: item.id }) }}"
                                   class=" btn btn-default btn-xs" data-toggle="tooltip"
                                   title="Publier la catégorie {{ item.name }}"><i class="fas fa-circle fa-2x color_red"></i></a>
                            {% endif %}
                        </td>
                    </tr>
                {% endfor %}

            </tbody>
        </table>
    </div>
{% endblock %}
