<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Marketing;

use CpCreation\VitiCore\Product\Model\Product;

interface PublishedProductReviewRatingCollectionProvider
{
    /**
     * @param iterable<Product> $products
     *
     * @return array<string, ProductReviewRatingSummary>
     */
    public function getPublishedRatingSummaries(
        iterable $products,
        ?\DateTimeImmutable $now = null,
    ): array;
}
