<!DOCTYPE html>
<html lang="{{ locale|default('fr') }}">
<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="x-apple-disable-message-reformatting">
    <title>{{ 'marketing.review.withdrawal_mail.subject'|trans({'%company%': company}, 'messages', locale|default('fr')) }}</title>
    <style>
        body, table, td, a {
            -webkit-text-size-adjust: 100%;
            -ms-text-size-adjust: 100%;
        }

        table {
            border-collapse: collapse !important;
        }

        img {
            border: 0;
            height: auto;
            line-height: 100%;
            outline: none;
            text-decoration: none;
            -ms-interpolation-mode: bicubic;
        }

        @media only screen and (max-width: 640px) {
            .email-shell { padding: 20px 12px !important; }
            .email-card { width: 100% !important; }
            .email-header, .email-body { padding-right: 22px !important; padding-left: 22px !important; }
            .email-header { padding-top: 32px !important; padding-bottom: 24px !important; }
            .email-body { padding-top: 26px !important; padding-bottom: 32px !important; font-size:17px !important; }
            .email-logo { max-width: 150px !important; margin-bottom: 20px !important; }
            .email-title { font-size: 28px !important; }
        }
    </style>
</head>
<body style="margin:0; padding:0; background:#f7f2ea; color:#201917; font-family:Arial, Helvetica, sans-serif;">
<table role="presentation" width="100%" cellspacing="0" cellpadding="0" style="width:100%; margin:0; padding:0; background:#f7f2ea;">
    <tr>
        <td class="email-shell" align="center" style="padding:32px 16px;">
            <table role="presentation" width="640" cellspacing="0" cellpadding="0" class="email-card" style="width:100%; max-width:640px; background:#ffffff;">
                <tr>
                    <td class="email-header" style="padding:40px 40px 28px; text-align:center; background:#fcfaf7;">
                        {% if contact and contact.image and contact.image.path %}
                            <img
                                src="{{ absolute_url(asset(contact.image.path)|imagine_filter('logo_email_filter')) }}"
                                alt="{{ contact.image.title ?: company }}"
                                width="180"
                                class="email-logo"
                                style="display:block; width:100%; max-width:180px; margin:0 auto 24px;"
                            >
                        {% endif %}
                        <div style="font-size:12px; letter-spacing:0.22em; text-transform:uppercase; color:#9d7c47;">{{ company }}</div>
                        <h1 class="email-title" style="margin:16px 0 0; font-family:Georgia, 'Times New Roman', serif; font-size:34px; line-height:1.25; font-weight:400; color:#201917;">
                            {{ 'marketing.review.withdrawal_mail.heading'|trans({}, 'messages', locale|default('fr')) }}
                        </h1>
                    </td>
                </tr>
                <tr>
                    <td class="email-body" style="padding:32px 40px 40px; font-size:16px; line-height:1.7; color:#5d4b41;">
                        <p style="margin:0 0 24px;">{{ 'marketing.review.withdrawal_mail.intro'|trans({'%product%': product_label}, 'messages', locale|default('fr')) }}</p>

                        <table role="presentation" width="100%" cellspacing="0" cellpadding="0" style="width:100%; margin:0 0 24px; background:#fcfaf7;">
                            <tr>
                                <td style="padding:18px 22px 6px; font-size:12px; letter-spacing:0.14em; text-transform:uppercase; color:#9d7c47;">
                                    {{ 'marketing.review.withdrawal_mail.reason'|trans({}, 'messages', locale|default('fr')) }}
                                </td>
                            </tr>
                            <tr>
                                <td style="padding:0 22px 18px; color:#201917;">
                                    {{ ('marketing.review.moderation.reason.' ~ reason_code)|trans({}, 'messages', locale|default('fr')) }}
                                    {% if reason_detail %}
                                        <div style="margin-top:8px; color:#5d4b41;">{{ reason_detail }}</div>
                                    {% endif %}
                                </td>
                            </tr>
                        </table>

                        <p style="margin:0; padding-top:20px; border-top:1px solid #e7ded4; font-size:14px; color:#806f65;">
                            {{ 'marketing.review.withdrawal_mail.footer'|trans({'%company%': company}, 'messages', locale|default('fr')) }}
                        </p>
                    </td>
                </tr>
            </table>
        </td>
    </tr>
</table>
</body>
</html>
