<?php

namespace CpCreation\VitiCore\Behavior;

use CpCreation\VitiCore\Media\Model\Video;

interface HasVideo
{
    public function getVideo(): ?Video;

    public function setVideo(?Video $video): void;
}
