'use strict';

// Compatibilité de build pour les projets qui référencent encore l'ancien
// identifiant VitiCore. Les nouvelles vues utilisent directement le contrôleur
// cpcreation-altcha fourni par @cpcreation/altcha-bundle.
export {default} from '@cpcreation/altcha-bundle/controllers/altcha_controller.js';
