<div
  id="alert"
  data-live-ignore
  {{ turbo_stream_listen('/notifier') }}
  class="fixed top-20 right-5 z-50 flex w-full max-w-xs flex-col"
  data-turbo-cache="false">
  {% for level, messages in app.flashes() %}
    {% for message in messages %}
      <twig:Notification
        message="{{ message }}"
        level="{{ level }}"
        class="animate-toast flex w-full max-w-xs items-center rounded-lg bg-foreground p-4 text-on-foreground/70 shadow dark:bg-foreground dark:text-on-foreground/70"
        wrapper:class="inline-flex items-center justify-center flex-shrink-0 text-{{ level }} rounded-lg icon-{{ level }}"
        message:class="ml-3 text-sm font-normal"
        close:class="hover:text-foreground -mx-1.5 -my-1.5 ml-auto inline-flex h-8 w-8 cursor-pointer p-1.5 text-on-foreground">
      </twig:Notification>
    {% endfor %}
  {% endfor %}
</div>
