<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Marketing\DTO;

use CpCreation\VitiCore\Marketing\Model\ProductReviewReport;
use Symfony\Component\Validator\Constraints as Assert;

final class ProductReviewReportSubmission
{
    #[Assert\NotBlank]
    #[Assert\Choice(callback: [ProductReviewReport::class, 'getAllowedReasons'])]
    public string $reason = '';

    #[Assert\Length(max: 1000)]
    public ?string $comment = null;
}
