<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Enrollment\Message;

final readonly class DeliverCatalogEnrollmentMessage
{
    public function __construct(public string $requestId) {}
}
