{% use '@CpCreationVitiCore/formFlow/stepList_blocks.html.twig' %}
{%- if flow.getStepLabels() is not empty -%}
    <div class="fuelux-wizard" data-target="#step-container">
        <ul class="wizard-steps">
            {% for stepLabel in flow.getStepLabels() %}
                <li{{ block('craue_flow_stepList_class') }} data-target="#step{{loop.index}}">
                    {%- if craue_isStepLinkable(flow, loop.index) -%}
                        <a href="{{ path(app.request.attributes.get('_route'), app.request.query.all
                            | merge(app.request.attributes.get('_route_params'))
                            | craue_addDynamicStepNavigationParameters(flow, loop.index)) }}">
                            {{- block('craue_flow_stepLabel') -}}
                        </a>
                    {%- else -%}
                        {{ block('craue_flow_stepLabel') }}
                    {%- endif -%}
                </li>

            {% endfor %}
        </ul>
        <div class="clearfix"></div>
        <br />
        <h3><strong>Etape {{ flow.currentStepNumber}} </strong> - {{ flow.currentStepLabel | trans }}</h3>
    </div>

{%- endif -%}
