{{ 'marketing.review.withdrawal_mail.heading'|trans({}, 'messages', locale|default('fr')) }}

{{ 'marketing.review.withdrawal_mail.intro'|trans({'%product%': product_label}, 'messages', locale|default('fr')) }}

{{ 'marketing.review.withdrawal_mail.reason'|trans({}, 'messages', locale|default('fr')) }} : {{ ('marketing.review.moderation.reason.' ~ reason_code)|trans({}, 'messages', locale|default('fr')) }}{% if reason_detail %} — {{ reason_detail }}{% endif %}

{{ 'marketing.review.withdrawal_mail.footer'|trans({'%company%': company}, 'messages', locale|default('fr')) }}
