<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogValidation
{
    /**
     * @param list<CatalogValidationIssue> $issues
     */
    public function __construct(
        public bool $valid,
        public array $issues,
    ) {}
}
