<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Seed;

interface SeedScopeProviderInterface
{
    /** @return \Traversable<string> project identifiers */
    public function iterateForSeeding(SeedPayload $seedPayload): \Traversable;
}
