<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Form\Admin;

use A2lix\TranslationFormBundle\Form\Type\TranslationsType;
use CpCreation\VitiCore\Product\Model\CommerceProductType;
use CpCreation\VitiCore\Product\Model\ProductCategory;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\EnumType;
use Symfony\Component\Form\Extension\Core\Type\IntegerType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

final class ProductCategoryType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add(
                'translations',
                TranslationsType::class,
                [
                    'label' => false,
                    'required_locales' => ['fr'],
                    'fields' => [
                        'name' => [
                            'field_type' => TextType::class,
                            'label' => 'Nom',
                            'required' => false,
                        ],
                        'description' => [
                            'field_type' => TrumbowygType::class,
                            'label' => 'description',
                            'required' => false,
                        ],
                        'noGroupMatchMessage' => [
                            'field_type' => TrumbowygType::class,
                            'label' => 'Texte à afficher dans la catégorie si le Group ne correspond pas pour inciter à se connecter ou à taper le code partenaire',
                            'required' => false,
                        ],
                        'shopCategoryMessage' => [
                            'field_type' => TrumbowygType::class,
                            'label' => 'Texte à afficher sous la catégorie uniquement dans la boutique',
                            'required' => false,
                        ],
                    ],
                    'excluded_fields' => ['slug'],
                ]
            )
            ->add('pageCuvee', CheckboxType::class, [
                'label' => 'Visualiser cette catégorie dans la page Cuvée',
                'required' => false,
            ])
            ->add('pageShop', CheckboxType::class, [
                'label' => 'Visualiser cette catégorie dans la page Boutique',
                'required' => false,
            ])
            ->add('displayEvenWithoutGroupMatch', CheckboxType::class, [
                'label' => 'Afficher quand même cette catégorie dans la boutique si elle contient au moins un produit, même si aucun produit ne correspond au groupe actif',
                'required' => false,
            ])
            ->add('commerceExportEnabled', CheckboxType::class, [
                'label' => 'product_category.commerce_export_enabled.label',
                'help' => 'product_category.commerce_export_enabled.help',
                'required' => false,
            ])
            ->add('commerceProductType', EnumType::class, [
                'class' => CommerceProductType::class,
                'label' => 'product_category.commerce_product_type.label',
                'help' => 'product_category.commerce_product_type.help',
                'choice_label' => static fn(CommerceProductType $type): string => 'product_category.commerce_product_type.' . $type->value,
            ])
            ->add('minimumPurchaseAge', IntegerType::class, [
                'label' => 'product_category.minimum_purchase_age.label',
                'help' => 'product_category.minimum_purchase_age.help',
                'required' => false,
                'attr' => [
                    'min' => 1,
                    'max' => 99,
                ],
            ]);
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults(
            [
                'data_class' => ProductCategory::class,
                'translation_domain' => 'form',
            ]
        );
    }
}
