<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogProductStatus
{
    /**
     * @param list<string> $reasons
     */
    public function __construct(
        public bool $published,
        public bool $orderable,
        public bool $price_visible,
        public bool $temporarily_paused,
        public array $reasons,
    ) {}
}
