<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogProductLocalization
{
    /**
     * @param list<string> $assemblages
     * @param list<string> $winemaking
     */
    public function __construct(
        public string $title,
        public string $name,
        public ?string $header,
        public ?string $quality,
        public ?string $slug,
        public CatalogRichText $description,
        public CatalogRichText $introduction,
        public CatalogRichText $shop_information,
        public ?string $dosage,
        public CatalogRichText $tasting,
        public array $assemblages,
        public array $winemaking,
    ) {}
}
