{% extends '@CpCreationVitiCore/admin/base.html.twig' %}

{% block content %}
    <div class="panel panel-default">
        <div class="panel-heading">
            Nom de ville à faire correspondre avec la réalité
        </div>
        <table class="table table-striped">
            <thead>
            <tr>
                <th>#</th>
                <th>Pays</th>
                <th>Departement</th>
                <th>Non formaté</th>
                <th>Code postal</th>
                <th>Ville</th>
                <th></th>
            </tr>
            </thead>
            <tbody>

            {% for item in citiesNoFound %}
                <tr>
                    <td>{{ loop.index }}</td>
                    <td>{{ item.country }}</td>
                    <td>{{ item.county }}</td>
                    <td>{{ item.oldName }}</td>
                    <td>{{ item.postal }}</td>
                    <td>{{ item.city }}</td>
                    <td>
                        <a href="{{ path('admin_shipping_auto_sup_edit', { id: item.id}) }}" class="btn btn-info btn-xs"
                           data-toggle="tooltip" title="Editer le nom"><i class="fas fa-edit fa-2x"></i></a>
                    </td>
                </tr>
            {% endfor %}
            </tbody>
        </table>
    </div>
    <div class="panel panel-default">
        <div class="panel-heading">
            Ville avec supplément de transport - Automatique
        </div>
        <table class="table table-striped">
            <thead>
            <tr>
                <th>#</th>
                <th>Pays</th>
                <th>Departement</th>
                <th>Non formaté</th>
                <th>Code postal</th>
                <th>Ville</th>
                <th></th>
            </tr>
            </thead>
            <tbody>

            {% for item in items %}
                <tr>
                    <td>{{ loop.index }}</td>
                    <td>{{ item.country }}</td>
                    <td>{{ item.county }}</td>
                    <td>{{ item.oldName }}</td>
                    <td>{{ item.postal }}</td>
                    <td>{{ item.city }}</td>
                    <td>
                        <a href="{{ path('admin_shipping_auto_sup_edit', { id: item.id}) }}" class="btn btn-info btn-xs"
                           data-toggle="tooltip" title="Editer le nom"><i class="fas fa-edit fa-2x"></i></a>
                    </td>
                </tr>
            {% endfor %}
            </tbody>
        </table>
    </div>
{% endblock %}
