<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Notification\Command;

use CpCreation\VitiCore\Bridge\Symfony\Bundle\Messenger\Messaging\CommandInterface;
use CpCreation\VitiCore\Bridge\Symfony\Bundle\Notification\Enum\PushChannel;
use CpCreation\VitiCore\Bridge\Symfony\Bundle\Notification\Enum\PushLevel;
use CpCreation\VitiCore\Bridge\Symfony\Bundle\Notification\Enum\PushTarget;
use CpCreation\VitiCore\Bridge\Symfony\Bundle\Notification\Enum\PushTransport;
use CpCreation\VitiCore\Bridge\Symfony\Bundle\Notification\Enum\PushType;

final class NotificationCommand implements CommandInterface
{
    public function __construct(
        public string $message,
        public PushChannel $channel = PushChannel::NOTIFIER,
        public PushTarget $target = PushTarget::WINDOW,
        public PushLevel $level = PushLevel::SUCCESS,
        public PushTransport $transport = PushTransport::MERCURE,
        public PushType $type = PushType::APP,
        public ?string $template = null,
    ) {
    }
}
