{{ 'security.mail.reset.greeting'|trans({}, null, locale|default('fr')) }}

{{ 'security.mail.reset.intro'|trans({'%company%': company }, null, locale|default('fr')) }}

{{ 'security.mail.reset.txt_instructions'|trans({}, null, locale|default('fr')) }}
{{ reset_url }}

{{ 'security.mail.reset.expiry'|trans({'%date%': expires_at|date('d/m/Y H:i') }, null, locale|default('fr')) }}

{{ 'security.mail.reset.ignore'|trans({}, null, locale|default('fr')) }}
