<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

use ApiPlatform\Metadata\ApiProperty;

final readonly class CatalogCompliance
{
    public function __construct(
        public bool $catalog_export_enabled,
        #[ApiProperty(
            openapiContext: ['enum' => ['unknown', 'general', 'alcohol', 'service']],
            jsonSchemaContext: ['enum' => ['unknown', 'general', 'alcohol', 'service']],
        )]
        public string $product_type,
        #[ApiProperty(
            openapiContext: ['minimum' => 1, 'maximum' => 99, 'example' => 18],
            jsonSchemaContext: ['minimum' => 1, 'maximum' => 99, 'example' => 18],
        )]
        public ?int $age_restriction,
        public bool $regulated,
    ) {}
}
