<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Product\DTO;

use CpCreation\VitiCore\Cart\Model\Cart;

final class DegressiveContext
{
    public function __construct(
        public readonly Cart $cart,
        public readonly string $priceGroupId,
        public readonly int|float $equivalencyQty,
    ) {
    }
}
