<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Bridge\Doctrine\Migrations;

use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;

final class Version20260721133200 extends AbstractMigration
{
    public function getDescription(): string
    {
        return 'Permet de desactiver individuellement l application du groupe importe lors de l inscription.';
    }

    public function up(Schema $schema): void
    {
        $this->addSql('ALTER TABLE user_imported_client_group ADD apply_on_registration BOOLEAN DEFAULT true NOT NULL');
    }

    public function down(Schema $schema): void
    {
        $this->addSql('ALTER TABLE user_imported_client_group DROP apply_on_registration');
    }
}
