<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Behavior\Impl;

use CpCreation\VitiCore\Product\Model\ProductPriceGroup;

trait GroupAvailable
{
    private ?ProductPriceGroup $groupAvailable = null;

    public function getGroupAvailable(): ?ProductPriceGroup
    {
        return $this->groupAvailable;
    }

    public function setGroupAvailable(?ProductPriceGroup $groupAvailable): void
    {
        $this->groupAvailable = $groupAvailable;
    }
}
