<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Bridge\Doctrine\Migrations;

use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;

final class Version20260604100000 extends AbstractMigration
{
    public function getDescription(): string
    {
        return 'Renomme la table SocialNetworkNew en SocialNetwork.';
    }

    public function up(Schema $schema): void
    {
        $this->addSql('ALTER TABLE IF EXISTS socialnetwork_socialnetworkNew RENAME TO socialnetwork_socialnetwork');
    }

    public function down(Schema $schema): void
    {
        $this->addSql('ALTER TABLE IF EXISTS socialnetwork_socialnetwork RENAME TO socialnetwork_socialnetworkNew');
    }
}
