<!DOCTYPE html>
<html lang="fr">
<head><meta charset="utf-8"><title>{{ 'withdrawal.mail.admin.heading'|trans({}, 'messages', 'fr') }}</title></head>
<body style="font-family:Arial, Helvetica, sans-serif; color:#222;">
    <h1>{{ 'withdrawal.mail.admin.heading'|trans({}, 'messages', 'fr') }}</h1>
    <p>{{ 'withdrawal.mail.admin.intro'|trans({}, 'messages', 'fr') }}</p>
    <ul>
        <li><strong>{{ 'withdrawal.summary.name'|trans({}, 'messages', 'fr') }} :</strong> {{ withdrawal_request.firstname }} {{ withdrawal_request.lastname }}</li>
        <li><strong>{{ 'withdrawal.summary.reference'|trans({}, 'messages', 'fr') }} :</strong> {{ withdrawal_request.contractReference }}</li>
        <li><strong>{{ 'withdrawal.summary.email'|trans({}, 'messages', 'fr') }} :</strong> {{ withdrawal_request.receiptEmail }}</li>
        <li><strong>{{ 'withdrawal.summary.order_received'|trans({}, 'messages', 'fr') }} :</strong> {{ (withdrawal_request.orderReceived is same as(true) ? 'withdrawal.form.yes' : (withdrawal_request.orderReceived is same as(false) ? 'withdrawal.form.no' : 'withdrawal.form.not_provided'))|trans({}, 'messages', 'fr') }}</li>
        {% if withdrawal_request.orderReceived and withdrawal_request.receivedAt %}
            <li><strong>{{ 'withdrawal.summary.received_at'|trans({}, 'messages', 'fr') }} :</strong> {{ withdrawal_request.receivedAt|date('d/m/Y') }}</li>
        {% endif %}
        {% if withdrawal_request.reasonCode %}
            <li><strong>{{ 'withdrawal.summary.reason'|trans({}, 'messages', 'fr') }} :</strong> {{ ('withdrawal.reason.' ~ withdrawal_request.reasonCode)|trans({}, 'messages', 'fr') }}</li>
        {% endif %}
        {% if withdrawal_request.comment %}
            <li><strong>{{ 'withdrawal.summary.comment'|trans({}, 'messages', 'fr') }} :</strong> {{ withdrawal_request.comment|nl2br }}</li>
        {% endif %}
        <li><strong>{{ 'withdrawal.summary.submitted_at'|trans({}, 'messages', 'fr') }} :</strong> {{ withdrawal_request.submittedAt|date('d/m/Y H:i:s T') }}</li>
    </ul>
</body>
</html>
