<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Product\DTO;

final class DegressiveResult
{
    /**
     * @param list<DegressiveAllocation> $allocations
     */
    public function __construct(
        public readonly int $amountTTC,
        public readonly array $allocations,
        public readonly bool $isBottleOffered = false,
    ) {
    }
}
