<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Marketing\DTO;

use Symfony\Component\Validator\Constraints as Assert;

final class ProductReviewSubmission
{
    #[Assert\NotBlank]
    public ?string $productId = null;

    #[Assert\Range(min: 1, max: 5)]
    public int $rating = 5;

    #[Assert\NotBlank]
    #[Assert\Length(min: 3, max: 5000)]
    public ?string $comment = null;
}
