<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogProductReviews
{
    /** @param list<CatalogProductReview> $items */
    public function __construct(
        public bool $enabled,
        public int $review_count,
        public ?float $average_rating,
        public array $items,
    ) {}
}
