<?php

namespace CpCreation\VitiCore\Product\DTO;

use CpCreation\VitiCore\Cart\Model\CartItem;

final class DegressiveAllocation
{
    public function __construct(
        public readonly CartItem $item,
        public readonly int $amountTTC,
        public readonly ?int $unitAmountTTC = null,
    ) {
    }
}
