<?php

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Controller\Site;

use CpCreation\VitiCore\Locale\Repository\LocaleRepository;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;

final class VerifyController extends AbstractController
{
    public function __construct(
        private readonly LocaleRepository $localeRepository,
    ) {
    }

    public function index(): Response
    {
        $locales = $this->localeRepository->findAll();

        return $this->render('verify.html.twig', [
            'locales' => $locales,
        ]);
    }
}
