<meta charset="UTF-8">
<title>{{ head_title|default('Welcome!') }}</title>

{{ head_meta|default('')|raw }}

{{ component('seo:SiteStructuredData', {
  pageTitle: head_title|default('')|striptags|trim
}) }}

{% if app.environment == 'dev' %}
  {{ encore_entry_link_tags("app", null, "front") }}
  {{ encore_entry_script_tags("app", null, "front") }}
{% else %}
  {% set scriptNonce = csp_nonce('script') %}
  {% set styleNonce = csp_nonce('style') %}
  {{ encore_entry_link_tags("app", null, "front", { nonce: styleNonce }) }}
  <meta name="csp-nonce" content="{{ scriptNonce }}">
  {{ encore_entry_script_tags("app", null, "front", { nonce: scriptNonce }) }}
{% endif %}

{{ head_stylesheets|default('')|raw }}
{{ head_javascripts|default('')|raw }}
{{ component('tracking:Scripts', { position: 'head' }) }}

<meta name="viewport" content="width=device-width, initial-scale=1.0">
<meta name="turbo-refresh-method" content="morph">
<meta name="turbo-prefetch" content="false">
<meta name="turbo-cache-control" content="no-cache">
<meta name="view-transition" content="same-origin">
<meta name="x-window-id" content="{{ app.request.headers.get('X-Window-Id') }}">
<meta name="x-mer" content="{{ app.request.headers.get('x-mer') }}">
