<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Enrollment;

final readonly class CatalogEnrollmentResponse
{
    /**
     * @param array<string, mixed> $raw
     */
    public function __construct(
        public string $instanceId,
        public string $sourceId,
        public string $webhookUrl,
        public string $status,
        public string $jwtIssuer,
        public string $jwtAudience,
        public string $jwtKeyId,
        public string $jwtPublicKeyPem,
        public array $raw,
    ) {}
}
