<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Shop\DTO;

final class CartQuantityProgressRule
{
    public function __construct(
        public readonly string $title,
        public readonly string $sourceLabel,
        public readonly string $requirementLabel,
        public readonly string $statusLabel,
        public readonly int $currentQuantity,
        public readonly int $targetQuantity,
        public readonly int $progressPercent,
        public readonly bool $isComplete,
    ) {
    }
}
