<?php
/**
 * @author Colinet Julien
 */

namespace CpCreation\VitiCore\Behavior;

interface HasTimestamp
{
    public function getCreatedAt(): \DateTimeInterface;

    public function getUpdatedAt(): \DateTimeInterface;

    public function setUpdatedAt(\DateTimeInterface $dateTime);
}