.product-gallery {
    position: relative;
}

.product-gallery .splide__pagination {
    bottom: -2rem;
    gap: 0.45rem;
}

.product-gallery .splide__pagination__page {
    width: 0.5rem;
    height: 0.5rem;
    margin: 0;
    background: rgba(173, 156, 118, 0.35);
    opacity: 1;
    transform: none;
    transition: background-color 200ms ease, transform 200ms ease;
}

.product-gallery .splide__pagination__page.is-active {
    background: #ad9c76;
    transform: scale(1.15);
}

.shop-product-gallery-main .splide__track.splide__track--slide.splide__track--ltr.splide__track--draggable {
    height: 100% !important;
}

.shop-product-gallery-thumb {
    width: 100%;
    height: 100%;
    background: transparent;
    padding: 0;
    overflow: hidden;
    transition: opacity 200ms ease;
    opacity: 0.65;
}

.shop-product-gallery-thumbnails .splide__slide.is-active .shop-product-gallery-thumb {
    opacity: 1;
}

.shop-product-gallery-main .splide__track {
    overflow: hidden;
}

.shop-product-gallery-thumbnails .splide__track {
    overflow: hidden;
}
