<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Enrollment;

final readonly class CatalogEnrollmentContext
{
    public function __construct(
        public string $instanceId,
        public string $configurationFingerprint,
        public CatalogEnrollmentPayload $payload,
    ) {}
}
