<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

use ApiPlatform\Metadata\ApiProperty;

final readonly class CatalogProductLifecycle
{
    public const string ACTIVE = 'active';
    public const string ARCHIVED = 'archived';
    public const string REMOVED = 'removed';

    public function __construct(
        #[ApiProperty(openapiContext: ['enum' => [self::ACTIVE, self::ARCHIVED, self::REMOVED]])]
        public string $status,
    ) {}
}
