{% extends 'base.html.twig' %}

{% block title %}{{ 'marketing.review.form.page_title'|trans }}{% endblock %}

{% block meta %}
    <meta name="robots" content="noindex,nofollow,noarchive">
    <meta name="referrer" content="no-referrer">
{% endblock %}

{% block main %}
    <section data-review-invitation-page class="mx-auto max-w-3xl px-4 py-16 sm:px-6 lg:px-8">
        <header class="mb-10 text-center">
            <p class="text-sm font-semibold uppercase tracking-widest">{{ 'marketing.review.form.verified_purchase'|trans }}</p>
            <h1 class="mt-3 text-3xl font-semibold">{{ 'marketing.review.form.heading'|trans }}</h1>
            <p class="mt-4">{{ 'marketing.review.form.intro'|trans }}</p>
        </header>

        <twig:marketing:ProductReviewForm :token="token" />

        <p class="mt-8 text-center text-sm">
            <a href="{{ path('shop_review_policy') }}">{{ 'marketing.review.policy.link'|trans }}</a>
        </p>
    </section>
{% endblock %}
