<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogMedia
{
    /**
     * @param array<string, string>      $alt_by_locale
     * @param list<CatalogMediaRendition> $renditions
     */
    public function __construct(
        public string $id,
        public string $source_id,
        public string $source_scope,
        public string $role,
        public int $position,
        public array $alt_by_locale,
        public CatalogMediaSource $source,
        public array $renditions,
    ) {}
}
