{% extends 'base.html.twig' %}

{% block title %}{% trans from 'meta' %}payment.title{% endtrans %}{% endblock %}

{% block meta %}
    {{ include('partials/seo.html.twig', {
        title_key: 'payment.title',
        description_key: 'payment.description',
        robots: 'noindex,follow'
    }) }}
{% endblock %}

{% block main %}
    <section class="bg-page py-10 sm:py-14">
        <div class="mx-auto max-w-5xl px-4 sm:px-6 lg:px-8">
            <twig:payment:Form :paymentId="app.request.attributes.get('id')" />
        </div>
    </section>
{% endblock %}
