<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Bridge\Doctrine\Migrations;

use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;

final class Version20260603125000 extends AbstractMigration
{
    public function getDescription(): string
    {
        return 'Supprime l ancien support SocialNetwork legacy au profit de SocialNetworkNew.';
    }

    public function up(Schema $schema): void
    {
        $this->addSql('DROP TABLE IF EXISTS socialnetwork_socialnetwork_audit CASCADE');
        $this->addSql('DROP TABLE IF EXISTS socialnetwork_socialnetwork CASCADE');
    }

    public function down(Schema $schema): void
    {
        $this->throwIrreversibleMigrationException('Le support SocialNetwork legacy a ete retire du code.');
    }
}
