<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Commerce\Catalog\Model;

enum CatalogChangeType: string
{
    case PRODUCT_CHANGED = 'product.changed';
    case PRODUCT_REMOVED = 'product.removed';
    case SELLER_CHANGED = 'seller.changed';
    case RECONCILIATION_REQUIRED = 'reconciliation.required';
}
