<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Enrollment;

interface CatalogHubRegistrar
{
    /**
     * Enregistre ou met à jour idempotemment une instance VitiCore dans le hub.
     */
    public function register(CatalogEnrollmentPayload $payload): CatalogEnrollmentResponse;
}
