<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogOption
{
    /**
     * @param array<string, CatalogOptionLocalization> $localizations
     * @param list<CatalogMedia>                       $media
     */
    public function __construct(
        public string $id,
        public string $source_id,
        public ?int $position,
        public array $localizations,
        public CatalogOptionPriceAdjustment $price_adjustment,
        public CatalogOptionAvailability $availability,
        public array $media,
    ) {}
}
