{% if orderingDisabledForVacation %}
    {{ 'shop.reopening_at'|trans({
        '%date%': vacationAt|format_datetime('long', 'short', locale: app.request.locale)
    }) }}
{% else %}
    {{ 'shop.view_with_partner_code'|trans }}
{% endif %}
