{% extends '@CpCreationVitiCore/admin/base.html.twig' %}

{% block content %}
    <div class="grid-container">
        <p><a href="{{ path('admin_social') }}">← {{ 'social_publishing.back'|trans }}</a></p>
        <h1>{{ 'social_publishing.form.title'|trans }}</h1>
        <p>{{ 'social_publishing.form.intro'|trans }}</p>

        <div class="panel panel-default">
            <div class="panel-body">
                {{ form_start(form) }}
                    {{ form_errors(form) }}
                    {{ form_row(form.channels) }}
                    {{ form_row(form.message) }}
                    {{ form_row(form.mediaUrl) }}
                    {{ form_row(form.scheduledAt) }}
                    {{ form_row(form.submit) }}
                {{ form_end(form) }}
            </div>
        </div>
    </div>
{% endblock %}
