<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Twig\Components\Shop;

use CpCreation\VitiCore\Bridge\Symfony\Bundle\Twig\Components\Cart\ItemForm;
use Symfony\UX\LiveComponent\Attribute\AsLiveComponent;

#[AsLiveComponent('shop:ProductCard', template: '@components/shop/ProductCard.html.twig')]
final class ProductCard extends ItemForm
{
}
