{% if contact.committeeChampagneSiret or contact.committeeChampagneIdentifiant %}
    <section class="border border-accent/20 bg-surface p-6 shadow-card sm:p-8">
        <h2 class="font-serif text-2xl uppercase tracking-card-title text-foreground">{{ 'legal_pages.agec.title'|trans }}</h2>
        <p class="mt-6 text-sm leading-7 text-foreground/80">{{ 'legal_pages.agec.intro'|trans }}</p>
        <ul class="mt-4 space-y-3 text-sm leading-7 text-foreground/80">
            {% if contact.committeeChampagneSiret %}
                <li>{{ 'legal_pages.agec.committee_champagne_siret'|trans({ '%value%': contact.committeeChampagneSiret }) }}</li>
            {% endif %}
            {% if contact.committeeChampagneIdentifiant %}
                <li>{{ 'legal_pages.agec.committee_champagne_identifier'|trans({ '%value%': contact.committeeChampagneIdentifiant }) }}</li>
            {% endif %}
        </ul>
    </section>
{% endif %}
