<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\User\Import;

final readonly class ClientImportResult
{
    public function __construct(
        public int $created,
        public int $updated,
        public int $unchanged,
        public int $ignored,
        public int $duplicatesInFile,
    ) {}
}
