<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogOptionPriceAdjustment
{
    /**
     * @param 'flat'|'per_unit' $mode
     */
    public function __construct(
        public int $amount_minor,
        public string $currency,
        public int $minor_unit_exponent,
        public string $mode,
    ) {}
}
