<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Enrollment;

/** Une demande préparée pour une ancienne autorisation ne doit plus affecter l'état courant. */
final class CatalogEnrollmentSupersededException extends \RuntimeException
{
}
