<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogVariantLocalization
{
    public function __construct(
        public string $title,
        public string $type_name,
        public string $label,
    ) {}
}
