{% extends '@CpCreationVitiCore/admin/base.html.twig' %}

{% block content %}
    <section id="forgot-password">
        <div class="container">
            <div class="row">
                <div class="col-md-6 col-md-offset-3">
                    <div class="panel panel-login">
                        <div class="panel-heading">
                            <div class="row">
                                <div class="col-xs-12">
                                    <a href="#" class="active">Mot de passe oublié</a>
                                </div>
                            </div>
                            <hr>
                        </div>
                        <div class="panel-body">
                            <p class="text-center">
                                Indiquez l'adresse email de votre compte administrateur pour recevoir un lien de réinitialisation.
                            </p>

                            {{ form_start(form) }}
                                <div class="form-group">
                                    {{ form_widget(form.email, {
                                        attr: {
                                            class: 'form-control',
                                            placeholder: 'Email',
                                            autocomplete: 'username'
                                        }
                                    }) }}
                                    {{ form_errors(form.email) }}
                                </div>

                                <div class="form-group">
                                    <div class="row">
                                        <div class="col-sm-8 col-sm-offset-2">
                                            <button type="submit" class="form-control btn btn-login">Envoyer le lien</button>
                                        </div>
                                    </div>
                                </div>

                                <div class="form-group">
                                    <div class="text-center">
                                        <a href="{{ path('login') }}">Retour à la connexion</a>
                                    </div>
                                </div>

                                <div class="hidden">{{ form_rest(form) }}</div>
                            {{ form_end(form) }}
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </section>
{% endblock content %}
