<?php

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Twig\Components\Behavior;

use Symfony\UX\LiveComponent\Attribute\LiveProp;

trait HasTemplateOverride
{
    #[LiveProp]
    public ?string $template = null;
}
