<?php

declare(strict_types=1);

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Messenger;

use CpCreation\VitiCore\Bridge\Symfony\Bundle\Event\DomainEvent;
use CpCreation\VitiCore\Bridge\Symfony\Bundle\Messenger\Messaging\EventBusInterface;
use Symfony\Component\Messenger\MessageBusInterface;

final readonly class EventBus implements EventBusInterface
{
    public function __construct(private MessageBusInterface $eventBus)
    {
    }

    public function publish(DomainEvent $event): void
    {
        $this->eventBus->dispatch($event);
    }
}
