<article data-information-block class="border-accent/35 mx-auto flex h-full max-w-xl flex-col border border-l-information border-l-accent/70 bg-information-card p-3 shadow-gold-soft backdrop-blur-sm sm:p-3.5">
    <div class="flex items-start gap-2.5 border-b border-accent/15 pb-2">
        {% if block.image and block.image.path %}
            <div class="flex h-10 w-10 shrink-0 items-center justify-center border border-accent/35 bg-accent/10 p-1.5 shadow-gold-action sm:h-11 sm:w-11">
                <twig:Image
                    alt="{{ block.title ?? 'Information' }}"
                    class="h-full w-full"
                    row:class="w-full"
                    img:class="h-full w-full object-contain"
                    lazy="{{ false }}"
                    quality="{{ 70 }}"
                    responsive="{{ false }}"
                    path="{{ block.image.path|image_path }}"
                    imageWidth="96"
                    imageHeight="96"
                />
            </div>
        {% endif %}
        <div class="min-w-0">
            <h3 class="font-serif text-sm leading-tight text-foreground sm:text-base">{{ block.title }}</h3>
            <div class="mt-1.5 h-0.5 w-8 bg-accent/70"></div>
        </div>
    </div>

    <div class="mt-2.5 flex-1 text-caption leading-4.5 text-foreground/85 sm:text-xs sm:leading-5 [&_p]:my-0">
        {{ block.description|raw }}
    </div>

    {% if block.actionType == constant('CpCreation\\VitiCore\\Shop\\Model\\InformationBlock::ACTION_LINK') and block.buttonLabel and block.buttonUrl %}
        <div class="mt-2.5">
            <a
                href="{{ block.buttonUrl }}"
                class="inline-flex items-center justify-center self-start border border-accent bg-accent px-2.5 py-1.5 text-micro font-semibold tracking-card-title uppercase text-foreground transition hover:bg-accent-soft sm:px-3 sm:text-caption"
            >
                {{ block.buttonLabel }}
            </a>
        </div>
    {% elseif block.actionType == constant('CpCreation\\VitiCore\\Shop\\Model\\InformationBlock::ACTION_MODAL') and block.buttonLabel and block.hasModalContent %}
        <div class="mt-2.5">
            <button
                type="button"
                class="inline-flex cursor-pointer items-center justify-center self-start border border-accent bg-accent px-2.5 py-1.5 text-micro font-semibold tracking-card-title uppercase text-foreground transition hover:bg-accent-soft sm:px-3 sm:text-caption"
                data-action="live#action"
                data-live-action-param="showInformationModal"
                data-live-information-block-id-param="{{ block.id.toString }}"
            >
                {{ block.buttonLabel }}
            </button>
        </div>
    {% endif %}
</article>
