<?php
/**
 * @author Colinet Julien
 */

namespace CpCreation\VitiCore\Behavior;

use Knp\DoctrineBehaviors\Contract\Entity\TranslatableInterface;

interface HasTranslatable extends TranslatableInterface
{

}
