{{ 'withdrawal.mail.admin.heading'|trans({}, 'messages', 'fr') }}

{{ 'withdrawal.mail.admin.intro'|trans({}, 'messages', 'fr') }}

{{ 'withdrawal.summary.name'|trans({}, 'messages', 'fr') }} : {{ withdrawal_request.firstname }} {{ withdrawal_request.lastname }}
{{ 'withdrawal.summary.reference'|trans({}, 'messages', 'fr') }} : {{ withdrawal_request.contractReference }}
{{ 'withdrawal.summary.email'|trans({}, 'messages', 'fr') }} : {{ withdrawal_request.receiptEmail }}
{{ 'withdrawal.summary.order_received'|trans({}, 'messages', 'fr') }} : {{ (withdrawal_request.orderReceived is same as(true) ? 'withdrawal.form.yes' : (withdrawal_request.orderReceived is same as(false) ? 'withdrawal.form.no' : 'withdrawal.form.not_provided'))|trans({}, 'messages', 'fr') }}
{% if withdrawal_request.orderReceived and withdrawal_request.receivedAt %}{{ 'withdrawal.summary.received_at'|trans({}, 'messages', 'fr') }} : {{ withdrawal_request.receivedAt|date('d/m/Y') }}
{% endif %}
{% if withdrawal_request.reasonCode %}{{ 'withdrawal.summary.reason'|trans({}, 'messages', 'fr') }} : {{ ('withdrawal.reason.' ~ withdrawal_request.reasonCode)|trans({}, 'messages', 'fr') }}
{% endif %}
{% if withdrawal_request.comment %}{{ 'withdrawal.summary.comment'|trans({}, 'messages', 'fr') }} : {{ withdrawal_request.comment }}
{% endif %}
{{ 'withdrawal.summary.submitted_at'|trans({}, 'messages', 'fr') }} : {{ withdrawal_request.submittedAt|date('d/m/Y H:i:s T') }}
