<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Marketing;

interface ProductReviewConfigurationProvider
{
    public function isEnabled(): bool;

    public function isModerationRequired(): bool;
}
