{% extends 'form_div_layout.html.twig' %}

{% block form_row %}
    <div class="host-project-form-row">
        {{ form_label(form) }}
        {{ form_widget(form) }}
        {{ form_errors(form) }}
    </div>
{% endblock %}
