{% extends '@CpCreationVitiCore/admin/base.html.twig' %}

{% block content %}
    <h1 class="page-header">Modifier la méthode de paiment {{ item.configuration.name }}</h1>
    {% include '@CpCreationVitiCore/formFlow/stepList_content.html.twig' %}

    {{ form_start(form, { 'attr': {'class': 'smart-form'} }) }}
    {{ form_errors(form) }}

    {% if flow.getCurrentStepNumber() == 2 %}
        <fieldset>
            <div class="small-12">
                {{ form_row(form.translations) }}
            </div>
        </fieldset>
    {% endif %}
    {{ form_rest(form) }}

    {% include '@CpCreationVitiCore/formFlow/buttons.html.twig' %}
    {{ form_end(form) }}
{% endblock %}