<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Twig;

use CpCreation\VitiCore\Contact\ContactProvider;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

final class ContactExtension extends AbstractExtension
{
    public function __construct(private readonly ContactProvider $contactProvider) {}

    public function getFunctions(): array
    {
        return [
            new TwigFunction('contact_business_name', $this->contactProvider->getBusinessName(...)),
        ];
    }
}
