<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Product\Model;

enum CommerceProductType: string
{
    case UNKNOWN = 'unknown';
    case GENERAL = 'general';
    case ALCOHOL = 'alcohol';
    case SERVICE = 'service';

    public function isRegulated(): bool
    {
        return self::ALCOHOL === $this;
    }
}
