<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Bridge\Doctrine\Migrations;

use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;

final class Version20260721133100 extends AbstractMigration
{
    public function getDescription(): string
    {
        return 'Aligne le nom de l index de groupe de la base clients importee avec le mapping Doctrine.';
    }

    public function up(Schema $schema): void
    {
        $this->addSql('ALTER INDEX idx_imported_client_group_group RENAME TO IDX_6CBBF7A3FE54D947');
    }

    public function down(Schema $schema): void
    {
        $this->addSql('ALTER INDEX IDX_6CBBF7A3FE54D947 RENAME TO idx_imported_client_group_group');
    }
}
