<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogCategory
{
    /**
     * @param array<string, CatalogCategoryLocalization> $localizations
     */
    public function __construct(
        public string $id,
        public string $source_id,
        public array $localizations,
    ) {}
}
