#cuvee_price_variants .form-group .actions {
    line-height: 80px;
}

.form-group {
    position: relative;
}

.color_green {
    color: #2b542c;
}

.color_red {
    color: #a83134;
}

.label-order-validated {
    background-color: #f6c27a;
    color: #6b4306;
}

label.required:before {
    content: "* ";
}

.trumbowyg-min .trumbowyg-box {
    min-height: 100px !important;
}

.trumbowyg-min .trumbowyg-editor {
    min-height: 100px !important;
}

.field-hide {
    display: none;
}

.panel-login {
    border-color: #ccc;
    -webkit-box-shadow: 0px 2px 3px 0px rgba(0, 0, 0, 0.2);
    -moz-box-shadow: 0px 2px 3px 0px rgba(0, 0, 0, 0.2);
    box-shadow: 0px 2px 3px 0px rgba(0, 0, 0, 0.2);
}

.panel-login > .panel-heading {
    color: #00415d;
    background-color: #fff;
    border-color: #fff;
    text-align: center;
}

.panel-login > .panel-heading a {
    text-decoration: none;
    color: #666;
    font-weight: bold;
    font-size: 15px;
    -webkit-transition: all 0.1s linear;
    -moz-transition: all 0.1s linear;
    transition: all 0.1s linear;
}

.panel-login > .panel-heading a.active {
    color: #029f5b;
    font-size: 18px;
}

.panel-login > .panel-heading hr {
    margin-top: 10px;
    margin-bottom: 0px;
    clear: both;
    border: 0;
    height: 1px;
    background-image: -webkit-linear-gradient(left, rgba(0, 0, 0, 0), rgba(0, 0, 0, 0.15), rgba(0, 0, 0, 0));
    background-image: -moz-linear-gradient(left, rgba(0, 0, 0, 0), rgba(0, 0, 0, 0.15), rgba(0, 0, 0, 0));
    background-image: -ms-linear-gradient(left, rgba(0, 0, 0, 0), rgba(0, 0, 0, 0.15), rgba(0, 0, 0, 0));
    background-image: -o-linear-gradient(left, rgba(0, 0, 0, 0), rgba(0, 0, 0, 0.15), rgba(0, 0, 0, 0));
}

.panel-login input[type="text"], .panel-login input[type="email"], .panel-login input[type="password"] {
    height: 45px;
    border: 1px solid #ddd;
    font-size: 16px;
    -webkit-transition: all 0.1s linear;
    -moz-transition: all 0.1s linear;
    transition: all 0.1s linear;
}

.panel-login input:hover,
.panel-login input:focus {
    outline: none;
    -webkit-box-shadow: none;
    -moz-box-shadow: none;
    box-shadow: none;
    border-color: #ccc;
}

.btn-login {
    background-color: #59B2E0;
    outline: none;
    color: #fff;
    font-size: 14px;
    height: auto;
    font-weight: normal;
    padding: 14px 0;
    text-transform: uppercase;
    border-color: #59B2E6;
}

.btn-login:hover,
.btn-login:focus {
    color: #fff;
    background-color: #53A3CD;
    border-color: #53A3CD;
}

.forgot-password {
    text-decoration: underline;
    color: #888;
}

.forgot-password:hover,
.forgot-password:focus {
    text-decoration: underline;
    color: #666;
}

[type='color'] {
    width: 25px;
    height: 25px;
    padding: 0;
}

.insta-default {
    margin: auto;
}

.insta-default a {
    padding: 15px 30px;
    display: block;
    background-color: #E33E5C;
    box-shadow: 0 2px 3px 0 rgba(0, 0, 0, 0.1);
    color: #FFF;
    border-radius: 4px;
    text-transform: uppercase;
    font-weight: bold;
    text-decoration: none;
    transition: all 0.3s;
    width: 33%;
    margin: 0 auto;
}

.insta-default a:hover {
    box-shadow: 0 0 4px 3px rgba(0, 0, 0, 0.15);
    transition: all 0.3s;
}

.insta-default a i {
    color: #FFF;
    font-size: 18px;
}

.contact {
    height: 35px;
    padding-top: 20px;
}

.contact a {
    display: block;
    text-decoration: none;
    color: #fff;
    border-radius: 10px;
    background-color: #bc2f2c;
    text-align: center;
    letter-spacing: .5px;
    margin: auto;
    width: 10%;
    transition: background-color .2s ease-out;
    cursor: pointer;
}


.btn-insta {
    text-decoration: none;
    background-color: #fff;
    padding: 1rem 2rem;
    text-decoration: none;
    color: #C71E7E;
    -webkit-transition: all .2s ease-in-out;
    transition: all .2s ease-in-out;
    margin-right: 10px;
}

.btn-social {
    margin: 0 auto;
    width: 174px;
    display: inline-block;
    height: 40px;
}

.btn-social:hover, .btn-social:focus {
    text-decoration: none;

}

.btn-insta i {
    color: #DF3796;
    font-size: 20px;
    padding-right: 10px;
    -webkit-transition: all .3s ease-in-out;
    transition: all .3s ease-in-out;
}

.btn-insta span {
    font-family: "Roboto", sans-serif;
    align-self: center;
    -webkit-transform: translateX(0px);
    transform: translateX(0px);
    -webkit-transition: all .1s ease-in-out;
    transition: all .1s ease-in-out;
    opacity: 1;
}

.btn-insta:hover {
    -webkit-transform: scale(1.1);
    transform: scale(1.1);
    background: -webkit-gradient(linear, left top, right top, from(#ff3019), to(#c90477));
    background: linear-gradient(to right, #ff3019 0%, #c90477 100%);
    box-shadow: 0px 10px 30px rgba(0, 0, 0, 0.1);
}

.btn-insta:hover i {
    -webkit-transform: translateX(45px);
    transform: translateX(45px);
    padding-right: 0;
    color: #FFF;
}

.btn-insta:hover span {
    -webkit-transform: translateX(30px);
    transform: translateX(30px);
    opacity: 0;
}

.btn-insta:active {
    -webkit-transform: scale(1);
    transform: scale(1);
    box-shadow: 0px 2px 10px rgba(0, 0, 0, 0.1);
}

.btn-f {
    min-width: 110px;
    width: 174px;
    background-color: #fff;
    padding: 1rem 2rem;
    text-decoration: none;
    color: #286ED6;
    -webkit-transition: all .2s ease-in-out;
    transition: all .2s ease-in-out;
}

.btn-f i {
    color: #286ED6;
    font-size: 20px;
    padding-right: 10px;
    -webkit-transition: all .3s ease-in-out;
    transition: all .3s ease-in-out;
}

.btn-f span {
    font-family: "Roboto", sans-serif;
    align-self: center;
    -webkit-transform: translateX(0px);
    transform: translateX(0px);
    -webkit-transition: all .1s ease-in-out;
    transition: all .1s ease-in-out;
    opacity: 1;
}

.btn-f:hover {
    -webkit-transform: scale(1.1);
    transform: scale(1.1);
    background-color: #286ED6;
    box-shadow: 0px 10px 30px rgba(0, 0, 0, 0.1);
}

.btn-f:hover i {
    -webkit-transform: translateX(45px);
    transform: translateX(45px);
    padding-right: 0;
    color: #FFF;
}

.btn-f:hover span {
    -webkit-transform: translateX(30px);
    transform: translateX(30px);
    opacity: 0;
}

.btn-f:active {
    -webkit-transform: scale(1);
    transform: scale(1);
    box-shadow: 0px 2px 10px rgba(0, 0, 0, 0.1);
}

.insta-photo {
    border-radius: 50%;
}

.btn-group a {
    margin-right: 20px;
}
