<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogSellerLink
{
    /**
     * @param array<string, string> $urls
     */
    public function __construct(
        public string $type,
        public array $urls,
    ) {}
}
