<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Marketing\Emailing;

final class BrevoApiException extends \RuntimeException
{
    public function __construct(
        string $message,
        public readonly ?int $statusCode = null,
        ?\Throwable $previous = null,
    ) {
        parent::__construct($message, 0, $previous);
    }
}
