{% set config = this.config %}

<div {{ attributes.defaults({ 'data-scroll-target': 'content', class: 'bg-page max-w-large large:mx-auto relative mx-2 flex flex-col lg:mx-8' }) }}>
    {% if config.enablePartnerCodeFront and config.showPartnerCodeWhenShopBlocked %}
        <div class="mt-10 px-4 md:px-10 lg:mt-20 lg:px-20">
            {{ component('shop:PartnerCode') }}
        </div>
    {% endif %}

    {% include 'shop/partials/blocked_notice.html.twig' with {
        config: config,
        blockedCountryCode: this.blockedCountryCode
    } only %}
</div>
