{% set pricing = cart.pricing %}
{{ 'order.resend_order_mail.intro'|trans({}, null, locale|default('fr')) }}

{{ 'order.resend_order_mail.reference_intro'|trans({'%document%': document_label_lower}, null, locale|default('fr')) }}

{{ 'order.resend_order_mail.reference_label'|trans({}, null, locale|default('fr')) }}: {{ reference }}

{{ 'order.resend_order_mail.total_label'|trans({}, null, locale|default('fr')) }}: {{ pricing ? pricing.totalTTC|price_format : cart.total|price_format }} €

{{ 'order.resend_order_mail.attachment_notice'|trans({'%document%': document_label_lower}, null, locale|default('fr')) }}

{{ 'order.resend_order_mail.footer'|trans({'%company%': company}, null, locale|default('fr')) }}
