{{ email_title }}

{{ 'marketing.abandoned_cart.mail.intro'|trans({'%company%': company}, 'messages', locale|default('fr')) }}

{{ 'marketing.abandoned_cart.mail.cart_heading'|trans({}, 'messages', locale|default('fr')) }}
{% for item in cart.items %}
{% set variant = item.variant %}
{% set product_name = variant ? (variant.fullName ?: (variant.product ? variant.product.fullName : '')) : '' %}
- {{ 'marketing.abandoned_cart.mail.cart_quantity'|trans({'%quantity%': item.quantity}, 'messages', locale|default('fr')) }} {{ item.finalPriceTTC|price_format }} € — {{ product_name ?: '-' }}{% if item.selectedOptionName %} — {{ 'shop.option_label'|trans({}, 'messages', locale|default('fr')) }} : {{ item.selectedOptionName }}{% endif %} — {{ item.totalFinalTTC|price_format }} €
{% endfor %}
{{ 'marketing.abandoned_cart.mail.cart_subtotal'|trans({}, 'messages', locale|default('fr')) }} : {{ cart_subtotal_ttc|price_format }} €

{% if has_existing_cart_discounts %}
{{ 'marketing.abandoned_cart.mail.existing_discounts_heading'|trans({}, 'messages', locale|default('fr')) }}
{% if cart_catalog_discount_ttc > 0 %}- {{ 'shop.discount'|trans({}, 'messages', locale|default('fr')) }} : -{{ cart_catalog_discount_ttc|price_format }} €
{% endif %}
{% if cart_degressive_discount_ttc > 0 %}- {{ (cart_degressive_is_bottle_offered ? 'shop.of_which_bottle_offered' : 'shop.of_which_degressive')|trans({}, 'messages', locale|default('fr')) }} : -{{ cart_degressive_discount_ttc|price_format }} €
{% endif %}
{% for cart_promo in existing_cart_promos %}
{% if cart_promo.discountItemsTTC > 0 %}- {{ 'shop.discount'|trans({}, 'messages', locale|default('fr')) }}{% if cart_promo.code %} — {{ cart_promo.code }}{% endif %}{% if cart_promo.percent is not null %} — {{ (cart_promo.percent * 100)|number_format(0, ',', ' ') }}%{% endif %} : -{{ cart_promo.discountItemsTTC|price_format }} €
{% endif %}
{% if cart_promo.discountShippingTTC > 0 %}- {{ 'shop.discount_shipping'|trans({}, 'messages', locale|default('fr')) }}{% if cart_promo.code %} — {{ cart_promo.code }}{% endif %} : -{{ cart_promo.discountShippingTTC|price_format }} €
{% endif %}
{% endfor %}
{% endif %}

{% if promo %}
{{ 'marketing.abandoned_cart.mail.coupon_heading'|trans({}, 'messages', locale|default('fr')) }} : {{ promo.title }}
{% if promo.amountTTC is not null %}{{ 'marketing.abandoned_cart.mail.fixed_discount'|trans({'%amount%': promo.amountTTC|price_format}, 'messages', locale|default('fr')) }}{% endif %}
{% if promo.percent is not null %}{{ (has_existing_cart_discounts ? 'marketing.abandoned_cart.mail.percent_discount_additional' : 'marketing.abandoned_cart.mail.percent_discount')|trans({'%percent%': (promo.percent * 100)|round(2)}, 'messages', locale|default('fr')) }}{% endif %}
{% if promo.percentShippingCost is not null %}{{ 'marketing.abandoned_cart.mail.free_shipping'|trans({}, 'messages', locale|default('fr')) }}{% endif %}
{{ 'marketing.abandoned_cart.mail.coupon_cart_change_warning'|trans({}, 'messages', locale|default('fr')) }}

{% endif %}
{{ 'marketing.abandoned_cart.mail.button'|trans({}, 'messages', locale|default('fr')) }} :
{{ recovery_url }}

{{ 'marketing.abandoned_cart.mail.expiry'|trans({'%date%': expires_at|date('d/m/Y')}, 'messages', locale|default('fr')) }}
