<!DOCTYPE html>
<html lang="{{ app.request.locale }}">
<head>
    {% set head_title %}{% block title %}Welcome!{% endblock %}{% endset %}
    {% set head_meta %}{% block meta %}{% endblock %}{% endset %}
    {% set head_stylesheets %}{% block stylesheets %}{% endblock %}{% endset %}
    {% set head_javascripts %}{% block javascripts %}{% endblock %}{% endset %}
    {{ include('partials/head.html.twig', {
        head_title: head_title,
        head_meta: head_meta,
        head_stylesheets: head_stylesheets,
        head_javascripts: head_javascripts
    }) }}
</head>
<body x-data="{ cartOpen: false}"
      :class="{ 'overflow-hidden': cartOpen }" @keydown.esc.window="cartOpen = false">
{{ component('tracking:Scripts', { position: 'body' }) }}

{% block header %}
    {{ component('announcement:Banner') }}
    {% block header_top %}{% endblock %}
    <header id="header">
    </header>
{% endblock header %}
<main id="main" class="relative w-full overflow-hidden">
    {% block main %} {% endblock %}
</main>
{% block footer %}
    <footer id="footer">
    </footer>
{% endblock %}


{% block mobile_bottom_bar %}{% endblock %}
{% include ('shop/partials/cart.html.twig') %}
{{ component('announcement:Modal') }}
{{ component('modal:SimpleModalDegressivity') }}
{{ component('modal:SimpleModalAward') }}
{{ component('modal:SimpleModalProductDetail') }}
{{ component('modal:SimpleModalProductReviews') }}
{{ component('modal:ShopInformation') }}
{{ include('partials/notifier.html.twig') }}
</body>
</html>
