<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Enrollment;

final class CatalogHubRegistrationException extends \RuntimeException
{
    /**
     * @param array<string, mixed>|null $response
     */
    public function __construct(
        string $message,
        public readonly bool $retryable,
        public readonly ?int $statusCode = null,
        public readonly ?array $response = null,
        ?\Throwable $previous = null,
    ) {
        parent::__construct($message, previous: $previous);
    }
}
