<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogQuantityRules
{
    public function __construct(
        public int $minimum,
        public int $increment,
        public ?int $maximum,
    ) {}
}
