<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Monitoring\Error\Message;

final readonly class DeliverErrorReportMessage
{
    /** @param array<string, mixed> $payload */
    public function __construct(public array $payload) {}
}
