<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Bridge\Doctrine\Migrations;

use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;

final class Version20260529124500 extends AbstractMigration
{
    public function getDescription(): string
    {
        return 'Ajoute la configuration des conversions Google Ads.';
    }

    public function up(Schema $schema): void
    {
        $this->addSql('ALTER TABLE shop_shop_config ADD COLUMN IF NOT EXISTS google_ads_conversion_labels JSON DEFAULT \'[]\'');
    }

    public function down(Schema $schema): void
    {
        $this->addSql('ALTER TABLE shop_shop_config DROP COLUMN IF EXISTS google_ads_conversion_labels');
    }
}
