<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogPrice
{
    public function __construct(
        public ?int $amount_minor,
        public ?int $regular_amount_minor,
        public ?int $discount_amount_minor,
        public string $currency,
        public int $minor_unit_exponent,
        public bool $tax_included,
        public bool $publicly_visible,
    ) {}
}
