{% extends '@CpCreationVitiCore/admin/base.html.twig' %}

{% block content %}
    <h1 class="page-header">{{ 'marketing.admin.title'|trans }}</h1>
    <p class="lead">{{ 'marketing.admin.intro'|trans }}</p>

    <div class="row">
        <div class="col-md-6">
            <div class="panel panel-info">
                <div class="panel-heading">
                    <i class="fas fa-shopping-cart"></i>
                    {{ 'marketing.abandoned_cart.admin.title'|trans }}
                </div>
                <div class="panel-body">
                    <p>{{ 'marketing.abandoned_cart.admin.description'|trans }}</p>
                    <p>
                        <strong>{{ 'marketing.abandoned_cart.admin.status'|trans }} :</strong>
                        {% if abandoned_cart_configuration and abandoned_cart_configuration.enabled %}
                            <span class="label label-success">{{ 'marketing.common.enabled'|trans }}</span>
                        {% else %}
                            <span class="label label-default">{{ 'marketing.common.disabled'|trans }}</span>
                        {% endif %}
                    </p>
                    <p>{{ 'marketing.abandoned_cart.admin.reminder_count'|trans({'%count%': abandoned_cart_reminder_count}) }}</p>
                    <a class="btn btn-primary" href="{{ path('admin_marketing_abandoned_cart') }}">
                        {{ 'marketing.abandoned_cart.admin.configure'|trans }}
                    </a>
                    <a class="btn btn-info" href="{{ path('admin_marketing_abandoned_cart_reminders') }}">
                        {{ 'marketing.abandoned_cart.admin.view_reminders'|trans }}
                    </a>
                </div>
            </div>
        </div>

        <div class="col-md-6">
            <div class="panel panel-info">
                <div class="panel-heading">
                    <i class="fas fa-envelope-open-text"></i>
                    {{ 'marketing.review_request.admin.title'|trans }}
                </div>
                <div class="panel-body">
                    <p>{{ 'marketing.review_request.admin.description'|trans }}</p>
                    <p>
                        <strong>{{ 'marketing.review_request.admin.status'|trans }} :</strong>
                        {% if configuration and configuration.enabled %}
                            <span class="label label-success">{{ 'marketing.common.enabled'|trans }}</span>
                        {% else %}
                            <span class="label label-default">{{ 'marketing.common.disabled'|trans }}</span>
                        {% endif %}
                    </p>
                    <p>{{ 'marketing.review_request.admin.invitation_count'|trans({'%count%': invitation_count}) }}</p>
                    <a class="btn btn-primary" href="{{ path('admin_marketing_review_request') }}">
                        {{ 'marketing.review_request.admin.configure'|trans }}
                    </a>
                </div>
            </div>
        </div>

        <div class="col-md-6">
            <div class="panel panel-info">
                <div class="panel-heading">
                    <i class="fas fa-star"></i>
                    {{ 'marketing.review.admin.title'|trans }}
                </div>
                <div class="panel-body">
                    <p>{{ 'marketing.review.admin.description'|trans }}</p>
                    <p>{{ 'marketing.review.admin.pending_count'|trans({'%count%': pending_review_count}) }}</p>
                    <a class="btn btn-primary" href="{{ path('admin_marketing_product_reviews') }}">
                        {{ 'marketing.review.admin.moderate'|trans }}
                    </a>
                </div>
            </div>
        </div>

        <div class="col-md-6">
            <div class="panel panel-info">
                <div class="panel-heading">
                    <i class="fas fa-flag"></i>
                    {{ 'marketing.review_report.admin.title'|trans }}
                </div>
                <div class="panel-body">
                    <p>{{ 'marketing.review_report.admin.description'|trans }}</p>
                    <p>{{ 'marketing.review_report.admin.open_count'|trans({'%count%': open_review_report_count}) }}</p>
                    <a class="btn btn-primary" href="{{ path('admin_marketing_product_review_reports') }}">
                        {{ 'marketing.review_report.admin.manage'|trans }}
                    </a>
                </div>
            </div>
        </div>
    </div>
{% endblock %}
