{% extends '@CpCreationVitiCore/admin/base.html.twig' %}

{% block content %}
    <section id="forgot-password-pending">
        <div class="container">
            <div class="row">
                <div class="col-md-6 col-md-offset-3">
                    <div class="panel panel-login">
                        <div class="panel-heading">
                            <div class="row">
                                <div class="col-xs-12">
                                    <a href="#" class="active">Email envoyé</a>
                                </div>
                            </div>
                            <hr>
                        </div>
                        <div class="panel-body">
                            <p class="text-center">
                                Si un compte administrateur existe{% if reset_pending_email %} pour <strong>{{ reset_pending_email }}</strong>{% endif %},
                                un lien de réinitialisation vient d'être envoyé.
                            </p>
                            <p class="text-center">
                                Ce lien expire au bout de 15 minutes.
                            </p>
                            <div class="text-center">
                                <a href="{{ path('login') }}">Retour à la connexion</a>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </section>
{% endblock content %}
