<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogPublishabilityReason
{
    public function __construct(
        public string $code,
        public ?string $path = null,
    ) {}
}
