{{ (resubmission
    ? 'marketing.review.submission_mail.resubmitted_heading'
    : 'marketing.review.submission_mail.heading')|trans({}, 'messages', locale|default('fr')) }}

{{ 'marketing.review.submission_mail.intro'|trans({
    '%author%': review.authorDisplayName,
    '%product%': product_label
}, 'messages', locale|default('fr')) }}

{{ 'marketing.review.submission_mail.details_heading'|trans({}, 'messages', locale|default('fr')) }}
{{ 'marketing.review.field.author'|trans({}, 'messages', locale|default('fr')) }} : {{ review.authorDisplayName }}
{{ 'marketing.review.field.product'|trans({}, 'messages', locale|default('fr')) }} : {{ product_label }}
{{ 'marketing.review.field.rating'|trans({}, 'messages', locale|default('fr')) }} : {{ review.rating }}/5
{{ 'marketing.review.field.date'|trans({}, 'messages', locale|default('fr')) }} : {{ review.submittedAt|date('d/m/Y H:i') }}
{% if review.title %}{{ 'marketing.review.form.title'|trans({}, 'messages', locale|default('fr')) }} : {{ review.title }}
{% endif %}
{{ 'marketing.review.field.comment'|trans({}, 'messages', locale|default('fr')) }} :
{{ review.comment }}

{{ 'marketing.review.submission_mail.neutrality_notice'|trans({}, 'messages', locale|default('fr')) }}
{% if publish_url or reject_url %}

{{ 'marketing.review.submission_mail.action_help'|trans({}, 'messages', locale|default('fr')) }}
{% if publish_url %}{{ 'marketing.review.submission_mail.publish_button'|trans({}, 'messages', locale|default('fr')) }} : {{ publish_url }}
{% endif %}
{% if reject_url %}{{ 'marketing.review.submission_mail.reject_button'|trans({}, 'messages', locale|default('fr')) }} : {{ reject_url }}
{% endif %}
{% else %}

{{ 'marketing.review.submission_mail.view_button'|trans({}, 'messages', locale|default('fr')) }} : {{ moderation_url }}
{% endif %}
