<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogOptionAvailability
{
    /**
     * @param 'in_stock'|'out_of_stock' $status
     */
    public function __construct(
        public string $status,
        public bool $orderable,
    ) {}
}
