{{ email_title }}

{{ (is_reminder ? 'marketing.review_request.mail.reminder_intro' : 'marketing.review_request.mail.intro')|trans({'%company%': company}, 'messages', locale|default('fr')) }}

{{ 'marketing.review_request.mail.order_heading'|trans({'%reference%': cart.token ?: cart.id}, 'messages', locale|default('fr')) }}
{{ 'marketing.review_request.mail.products_heading'|trans({}, 'messages', locale|default('fr')) }}
{% for item in cart.items %}
{% set variant = item.variant %}
{% set product_name = variant ? (variant.fullName ?: (variant.product ? variant.product.fullName : '')) : '' %}
{% if product_name %}
- {{ 'marketing.review_request.mail.product_line'|trans({
    '%quantity%': item.quantity,
    '%product%': product_name,
}, 'messages', locale|default('fr')) }}
{% endif %}
{% endfor %}

{{ 'marketing.review_request.mail.button'|trans({}, 'messages', locale|default('fr')) }} :
{{ review_url }}

{% if google_business_review_url %}
{{ 'marketing.review_request.mail.google_intro'|trans({}, 'messages', locale|default('fr')) }}
{{ 'marketing.review_request.mail.google_button'|trans({}, 'messages', locale|default('fr')) }} :
{{ google_business_review_url }}

{% endif %}
{{ 'marketing.review_request.mail.expiry'|trans({'%date%': expires_at|date('d/m/Y')}, 'messages', locale|default('fr')) }}

{{ 'marketing.review_request.mail.policy_link'|trans({}, 'messages', locale|default('fr')) }} :
{{ review_policy_url }}

{{ 'marketing.review_request.mail.opt_out_link'|trans({}, 'messages', locale|default('fr')) }} :
{{ opt_out_url }}
