{{ 'order.credit_mail.intro'|trans({}, null, locale|default('fr')) }}

{{ 'order.credit_mail.reference_intro'|trans({}, null, locale|default('fr')) }}

{{ 'order.credit_mail.reference_label'|trans({}, null, locale|default('fr')) }}: {{ credit_reference ?: reference }}

{{ 'order.credit_mail.attachment_notice'|trans({'%document%': document_label_lower }, null, locale|default('fr')) }}

{% if credit_details.totals.total_ttc is defined %}
{{ 'order.credit_mail.total_label'|trans({}, null, locale|default('fr')) }}: {{ credit_details.totals.total_ttc|price_format }} €
{% endif %}

{% if refund_status %}
{{ ('order.credit_mail.refund_' ~ refund_status)|trans({}, null, locale|default('fr')) }}
{% endif %}

{{ 'order.credit_mail.footer'|trans({'%company%': company }, null, locale|default('fr')) }}
