<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Scheduler;

final readonly class DispatchReviewInvitationsMessage
{
    public function __construct(public int $limit = 100) {}
}
