<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Bridge\Symfony\Bundle\Form\Admin\DataTransformer;

use Symfony\Component\Serializer\SerializerInterface;

final class JsonModelTransformerFactory
{
    public function __construct(
        private readonly SerializerInterface $serializer,
    ) {
    }

    public function for(string $className): JsonModelTransformer
    {
        return new JsonModelTransformer($this->serializer, $className);
    }
}
