<?php

declare(strict_types=1);

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiCore\Commerce\Catalog\Model;

enum CatalogDeliveryStatus: string
{
    case PENDING = 'pending';
    case QUEUED = 'queued';
    case DELIVERED = 'delivered';
    case FAILED_PERMANENT = 'failed_permanent';
}
