<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogMediaSource
{
    public function __construct(
        public ?string $mime_type,
        public ?int $width,
        public ?int $height,
        public ?int $size_bytes,
    ) {}
}
