<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Marketing;

final readonly class ProductReviewCardSummary
{
    public function __construct(
        public int $reviewCount,
        public ?float $ratingValue,
    ) {}
}
