<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Enrollment;

use CpCreation\VitiCore\Commerce\Catalog\Enrollment\Model\CatalogEnrollmentRequest;

interface CatalogEnrollmentPayloadProvider
{
    public function currentContext(): ?CatalogEnrollmentContext;

    public function create(CatalogEnrollmentRequest $request): CatalogEnrollmentPayload;
}
