<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Monitoring\External\Message;

final readonly class DeliverExternalConnectionHealthMessage
{
    public function __construct(
        public array $payload,
    ) {}
}
