<?php
/**
 * @author Colinet Julien
 */
namespace CpCreation\VitiCore\Media\Repository;

use CpCreation\VitiCore\Media\Model\Media;
use CpCreation\VitiCore\Repository\BaseRepository;
use Doctrine\Persistence\ManagerRegistry;

class MediaRepository extends BaseRepository
{
    /**
     * MediaRepository constructor.
     * @param ManagerRegistry $registry
     * @throws \ReflectionException
     */
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Media::class);
    }
}