{% extends 'base.html.twig' %}

{% block title %}{{ 'find-us.title'|trans({}, 'meta') }}{% endblock %}

{% block meta %}
    {{ include('partials/seo.html.twig', {
        title_key: 'find-us.title',
        description_key: 'find-us.description',
        robots: 'noindex,nofollow'
    }) }}
{% endblock meta %}

{% block main %}
    <section class="pt-4 bg-page">
        <div class="mx-4 lg:mx-8 large:mx-auto max-w-large pb-20 pt-16 sm:pb-28 sm:pt-20">
            <div class="mx-auto mb-10 max-w-4xl px-6 text-center sm:px-8">
                <h1 class="flex flex-col items-center font-serif text-3xl uppercase tracking-label text-foreground sm:text-4xl lg:text-5xl after:mt-5 after:h-0.5 after:w-14 after:bg-accent">
                    {{ 'find-us'|trans({}, 'menu')|raw }}
                </h1>
                <p class="mt-5 text-sm leading-7 text-foreground/70 sm:text-base">
                    {{ 'find-us.description'|trans({}, 'meta') }}
                </p>
            </div>

            <div class="mx-auto max-w-7xl">
                <twig:depot />
            </div>
        </div>
    </section>
{% endblock %}
