<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\User\Import;

final readonly class ClientImportAnalysis
{
    /**
     * @param list<ClientImportRow> $rows
     * @param list<string>          $sourceGroups
     */
    public function __construct(
        public int $headerLine,
        public string $emailColumn,
        public string $emailHeader,
        public string $groupColumn,
        public string $groupHeader,
        public array $rows,
        public array $sourceGroups,
        public int $totalRows,
        public int $invalidEmailRows,
        public int $missingGroupRows,
    ) {}
}
