<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\User\Import;

final readonly class ClientImportRow
{
    public function __construct(
        public int $line,
        public string $email,
        public string $sourceGroup,
    ) {}
}
