<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\Commerce\Catalog\Api\Resource;

final readonly class CatalogSeller
{
    /**
     * @param list<CatalogSellerLink> $links
     */
    public function __construct(
        public string $id,
        public string $source_contact_id,
        public string $display_name,
        public ?string $legal_name,
        public string $store_country,
        public CatalogSellerAddress $address,
        public ?string $website_url,
        public array $links,
    ) {}
}
