<?php

declare(strict_types=1);

namespace CpCreation\VitiCore\SocialPublishing;

final readonly class Channel
{
    /** @param array<string, mixed> $capabilities */
    public function __construct(
        public string $id,
        public string $provider,
        public string $externalId,
        public string $name,
        public string $type,
        public array $capabilities,
    ) {}
}
